/*
** Big Crunch
** Compresseur exploitable et exprimental
**
** Copyright (C) 2005-2006 - KaBoom
**
** http://bcrunch.online.fr/
*/

#include"ArTGA.hpp"
#include"../Console/Commande.hpp"

namespace BigCrunch
{
    ArTGA::ArTGA(const char *nomFichier, ModeCodeur mode) : Codeur(nomFichier, mode)
    {
        _borneInf = 0;
        _borneSup = 0xffffffff;
    }

    ArTGA::ArTGA(const char *nomFichier, ModeCodeur mode, const char *nomSortie) : Codeur(nomFichier, mode, nomSortie)
    {
        _borneInf = 0;
        _borneSup = 0xffffffff;
    }

    //Gerer le preparer
#ifndef SFX
    const char *ArTGA::compresser()
    {
        //////////////////////////////////////////////////////////////
        //test si c'est bien un fichier tga valide non compresser et pas indexe
        EFichierMauvais MF;
        int imageType;
        int pixelDepth;
        int largeurImage, hauteurImage;

        try
        {
            U8 test;

            //est-ce qu'il y a une palette de couleur (couleur indexe)?
            _entree->avancer(1);
            test = _entree->lireOctet();
            if (test)
            {
                throw MF;
            }

            //est-ce que l'image est compresse, couleur indexe?
            imageType = _entree->lireOctet();
            if ((imageType == 0) || (imageType == 1) || (imageType == 9) || (imageType == 11))
            {
                throw MF;
            }

            //recup la largeur et la hauteur
            _entree->avancer(9);
            largeurImage = _entree->lireU16();
            hauteurImage = _entree->lireU16();

            //si image en degrade de gris: est du 8 ou 16 bits? si 8 bits alors mauvais
            pixelDepth = _entree->lireOctet();
            if (((imageType == 3) && (pixelDepth == 8)) || ((imageType == 10) && (pixelDepth == 16)))
            {
                throw MF;
            }

            //ecrire dans la sortie le header du fichier
            _entree->rembobiner();
            for (int i = 0; i < 18; ++i)
            {
                _sortie->ecrireOctet(_entree->lireOctet());
            }
        }
        catch (...)
        {
            throw MF;
        }
        //////////////////////////////////////////////////////////////

        //choix de l'encodqge selon 16, 24 ou 32 bits // gris ou pas // alpha ou pas
        switch (pixelDepth)
        {
            case 16:
				if (imageType == 3) //image en degrade de gris
				{
					//printf("image 16 bits degrade de gris\n");
					ArTGAgray16Compr(largeurImage, hauteurImage);
				}
				else
				{
					//printf("image 16 bits\n");
					ArTGA16Compr(largeurImage, hauteurImage);
				}
				break;
            case 24:
				if (imageType == 10) // image compresser en rle
				{
					//printf("image 24 bits rle\n");
					ArTGA24RLECompr(largeurImage, hauteurImage);
				}
				else
				{
					//printf("image 24 bits\n");
					ArTGA24Compr(largeurImage, hauteurImage);
				}
				break;
            case 32:
				if (imageType == 10) // image compresser en rle
				{
					//printf("image 32 bits rle\n");
					ArTGA32RLECompr(largeurImage, hauteurImage);
				}
				else
				{
					//printf("image 32bits\n");
					ArTGA32Compr(largeurImage, hauteurImage);
				}
				break;
            default:
				//printf("pas un pixelDepth valide\n");
				throw MF;
				break;
        }

        return _nomSortie;
    }
#endif
    const char *ArTGA::decompresser()
    {
        U8 imageType;
        U8 pixelDepth;
        int largeurImage, hauteurImage;

        //////////////////////////////////////////////////////////////////////
        //ecrit le header du tga dans le fichier de sortie et recup la profondeur de couleur
        //recup le type d'image pour savoir comment la decompresser
        _entree->avancer(2);
        imageType = _entree->lireOctet();

        //recup la largeur et la hauteur
        _entree->avancer(9);
        largeurImage = _entree->lireU16();
        hauteurImage = _entree->lireU16();

        //recup la profondeur de couleur de l'image pour comme au dessus...
        pixelDepth = _entree->lireOctet();

        _entree->rembobiner();
        for (int i = 0; i < 18; ++i)
        {
            _sortie->ecrireOctet(_entree->lireOctet());
        }
        //////////////////////////////////////////////////////////////////////

        //choix de l'encodqge selon 16, 24 ou 32 bits // gris ou pas // alpha ou pas
        switch (pixelDepth)
        {
            case 16:
				if (imageType == 3) //image en degrade de gris
				{
					//printf("image 16 bits gris\n");
					ArTGAgray16Decompr(largeurImage, hauteurImage);
				}
				else
				{
					//printf("image 16 bits\n");
					ArTGA16Decompr(largeurImage, hauteurImage);
				}
				break;
            case 24:
				if (imageType == 10) // image compresser en rle
				{
					//printf("image 24 bits rle\n");
					ArTGA24RLEDecompr(largeurImage, hauteurImage);
				}
				else
				{
					//printf("image 24 bits\n");
					ArTGA24Decompr(largeurImage, hauteurImage);
				}
				break;
            case 32:
				if (imageType == 10) // image compresser en rle
				{
					//printf("image 32 bits rle\n");
					ArTGA32RLEDecompr(largeurImage, hauteurImage);
				}
				else
				{
					//printf("image 32 bits\n");
					ArTGA32Decompr(largeurImage, hauteurImage);
				}
				break;
            default:
				printf("pas un pixelDepth valide\n");
				break;
        }

        return _nomSortie;
    }

#ifndef SFX
    void ArTGA::ArTGA32Compr(int largeurImage, int hauteurImage)
    {
        U8  buffer;

        //init les variables de parcours du fichier
        int composante = 0;

        while (composante < (largeurImage * hauteurImage * 4))
        {
            try
            {
                buffer = _entree->lireOctet();
            }
            catch (EFinFichier &)
			{
				break;
			}

			switch (composante % 4)
			{
				case 0:
					for ( int i = 7; i >= 0; --i)
					{
						if ((buffer >> i) & 1) //si le bit courant est un 1
						{
							_borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P0.p());
							P0.apprendre(1);
						}
						else
						{
							_borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P0.p()) + 1;
							P0.apprendre(0);
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_sortie->ecrireOctet((U8)(_borneInf >> 24));
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
						}
					}
					break;
				case 1:
					for ( int i = 7; i >= 0; --i)
					{
						if ((buffer >> i) & 1) //si le bit courant est un 1
						{
							_borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P1.p());
							P1.apprendre(1);
						}
						else
						{
							_borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P1.p()) + 1;
							P1.apprendre(0);
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_sortie->ecrireOctet((U8)(_borneInf >> 24));
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
						}
					}
					break;
				case 2:
					for ( int i = 7; i >= 0; --i)
					{
						if ((buffer >> i) & 1) //si le bit courant est un 1
						{
							_borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P2.p());
							P2.apprendre(1);
						}
						else
						{
							_borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P2.p()) + 1;
							P2.apprendre(0);
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_sortie->ecrireOctet((U8)(_borneInf >> 24));
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
						}
					}
					break;
				case 3:
					for ( int i = 7; i >= 0; --i)
					{
						if ((buffer >> i) & 1) //si le bit courant est un 1
						{
							_borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P3.p());
							P3.apprendre(1);
						}
						else
						{
							_borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P3.p()) + 1;
							P3.apprendre(0);
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_sortie->ecrireOctet((U8)(_borneInf >> 24));
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
						}
					}
					break;
			}

            ++composante;
        }

        _sortie->ecrireOctet((U8)(_borneSup  >> 24));
    }
#endif
    void ArTGA::ArTGA32Decompr(int largeurImage, int hauteurImage)
    {
        U16  buffer;
        U32  pivot,
        reel;
        int  i;

        i     = 0;
        reel   = 0;
        buffer = 0;

        while (i < 4)
        {
            try
            {
                buffer = _entree->lireOctet();
            }
            catch (EFinFichier &)
            {
                buffer = 0;
            }
            reel = (reel << 8) + buffer;
            buffer = 0;
            ++i;
        }

        int composante = 0;

        while (composante < (largeurImage * hauteurImage * 4))
        {
            switch (composante % 4)
            {
                case 0:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P0.p());
						if ( reel <= pivot)
						{
							_sortie->ecrireBit(1);
							P0.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							_sortie->ecrireBit(0);
							P0.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch(EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
                case 1:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P1.p());
						if ( reel <= pivot)
						{
							_sortie->ecrireBit(1);
							P1.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							_sortie->ecrireBit(0);
							P1.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch(EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
                case 2:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P2.p());
						if ( reel <= pivot)
						{
							_sortie->ecrireBit(1);
							P2.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							_sortie->ecrireBit(0);
							P2.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch(EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
                case 3:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P3.p());
						if ( reel <= pivot)
						{
							_sortie->ecrireBit(1);
							P3.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							_sortie->ecrireBit(0);
							P3.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch(EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
            }

            ++composante;
        }
    }

#ifndef SFX
    ////////////////////////////////////////////////////////////////////////////
    void ArTGA::ArTGA24Compr(int largeurImage, int hauteurImage)
    {
        U8  buffer;

        //init les variables de parcours du fichier
        int composante = 0;

        while (composante < (largeurImage * hauteurImage * 3))
        {
            try
            {
                buffer = _entree->lireOctet();
            }
            catch (EFinFichier &)
            {
                break;
            }

            switch (composante % 3)
            {
                case 0:
                for ( int i = 7; i >= 0; --i)
                {
                    if ((buffer >> i) & 1) //si le bit courant est un 1
                    {
                        _borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P0.p());
                        P0.apprendre(1);
                    }
                    else
                    {
                        _borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P0.p()) + 1;
                        P0.apprendre(0);
                    }
                    while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
                    {
                        _sortie->ecrireOctet((U8)(_borneInf >> 24));
                        _borneInf <<= 8;
                        _borneSup = (_borneSup << 8) + 0x000000ff;
                    }
                }
                break;
                case 1:
                for ( int i = 7; i >= 0; --i)
                {
                    if ((buffer >> i) & 1) //si le bit courant est un 1
                    {
                        _borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P1.p());
                        P1.apprendre(1);
                    }
                    else
                    {
                        _borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P1.p()) + 1;
                        P1.apprendre(0);
                    }
                    while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
                    {
                        _sortie->ecrireOctet((U8)(_borneInf >> 24));
                        _borneInf <<= 8;
                        _borneSup = (_borneSup << 8) + 0x000000ff;
                    }
                }
                break;
                case 2:
                for ( int i = 7; i >= 0; --i)
                {
                    if ((buffer >> i) & 1) //si le bit courant est un 1
                    {
                        _borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P2.p());
                        P2.apprendre(1);
                    }
                    else
                    {
                        _borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P2.p()) + 1;
                        P2.apprendre(0);
                    }
                    while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
                    {
                        _sortie->ecrireOctet((U8)(_borneInf >> 24));
                        _borneInf <<= 8;
                        _borneSup = (_borneSup << 8) + 0x000000ff;
                    }
                }
                break;
            }
            ++composante;
        }

        while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
        {
            _sortie->ecrireOctet((U8)(_borneInf >> 24));
            _borneInf <<= 8;
            _borneSup = (_borneSup << 8) + 0x000000ff;
        }

        _sortie->ecrireOctet((U8)(_borneSup  >> 24));
    }

#endif
    void ArTGA::ArTGA24Decompr(int largeurImage, int hauteurImage)
    {
        U16  buffer;
        U32  pivot,
        reel;
        int  i;

        i     = 0;
        reel   = 0;
        buffer = 0;

        while (i < 4)
        {
            try
            {
                buffer = _entree->lireOctet();
            }
            catch (EFinFichier &)
            {
                buffer = 0;
            }
            reel = (reel << 8) + buffer;
            buffer = 0;
            ++i;
        }

        int composante = 0;

        while (composante < (largeurImage * hauteurImage * 3))
        {
            switch (composante % 3)
            {
                case 0:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P0.p());
						if ( reel <= pivot)
						{
							_sortie->ecrireBit(1);
							P0.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							_sortie->ecrireBit(0);
							P0.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch(EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
                case 1:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P1.p());
						if ( reel <= pivot)
						{
							_sortie->ecrireBit(1);
							P1.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							_sortie->ecrireBit(0);
							P1.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch(EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
                case 2:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P2.p());
						if ( reel <= pivot)
						{
							_sortie->ecrireBit(1);
							P2.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							_sortie->ecrireBit(0);
							P2.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch(EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
            }

            ++composante;
		}

        U8 sign[] =
            {
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0x52,0x55,0x45,0x56,0x49,0x53,0x49,0x4F,0x4E,0x2D,0x58,0x46,0x49,0x4C,0x45,0x2E,0x00
            };
        _sortie->ecrireOctets(sign, 26);
    }
#ifndef SFX
    ////////////////////////////////////////////////////////////////////////////
    void ArTGA::ArTGA16Compr(int largeurImage, int hauteurImage)
    {
        U16  buffer;

        for (int i = 0; i < (largeurImage * hauteurImage); ++i)
        {
            try
            {
                buffer = _entree->lireU16();
            }
            catch (EFinFichier &)
            {
                break;
            }

            //printf("buffer : %0x\n", buffer);

            // composante 0
            for (int i = 14; i >= 10; --i)
            {
                if ((buffer >> i) & 1) //si le bit courant est un 1
                {
                    _borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P0.p());
                    P0.apprendre(1);
                }
                else
                {
                    _borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P0.p()) + 1;
                    P0.apprendre(0);
                }
                while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
                {
                    _sortie->ecrireOctet((U8)(_borneInf >> 24));
                    _borneInf <<= 8;
                    _borneSup = (_borneSup << 8) + 0x000000ff;
                }
            }

            // composante 1
            for (int i = 9; i >= 5; --i)
            {
                if ((buffer >> i) & 1) //si le bit courant est un 1
                {
                    _borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P1.p());
                    P1.apprendre(1);
                }
                else
                {
                    _borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P1.p()) + 1;
                    P1.apprendre(0);
                }
                while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
                {
                    _sortie->ecrireOctet((U8)(_borneInf >> 24));
                    _borneInf <<= 8;
                    _borneSup = (_borneSup << 8) + 0x000000ff;
                }
            }

            // composante 2
            for (int i = 4; i >= 0; --i)
            {
                if ((buffer >> i) & 1) //si le bit courant est un 1
                {
                    _borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P2.p());
                    P2.apprendre(1);
                }
                else
                {
                    _borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P2.p()) + 1;
                    P2.apprendre(0);
                }
                while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
                {
                    _sortie->ecrireOctet((U8)(_borneInf >> 24));
                    _borneInf <<= 8;
                    _borneSup = (_borneSup << 8) + 0x000000ff;
                }
            }
        }

        while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
        {
            _sortie->ecrireOctet((U8)(_borneInf >> 24));
            _borneInf <<= 8;
            _borneSup = (_borneSup << 8) + 0x000000ff;
        }

        _sortie->ecrireOctet((U8)(_borneSup  >> 24));
    }
#endif
    void ArTGA::ArTGA16Decompr(int largeurImage, int hauteurImage)
    {
        U16  buffer;
        U32  pivot,
        reel;
        int  i;

        i     = 0;
        reel   = 0;
        buffer = 0;

        while (i < 4)
        {
            try
            {
                buffer = _entree->lireOctet();
            }
            catch (EFinFichier &)
            {
                buffer = 0;
            }
            reel = (reel << 8) + buffer;
            buffer = 0;
            ++i;
        }

        U16 bufferEcriture = 0;

		for (int i = 0; i < (largeurImage * hauteurImage); ++i)
        {
            bufferEcriture = 0;

            // composante 0
            for (int i = 0; i < 5; ++i)
            {
                pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P0.p());
                if ( reel <= pivot)
                {
                    bufferEcriture <<= 1;
                    bufferEcriture += 1;
                    P0.apprendre(1);
                    _borneSup = pivot;
                }
                else
                {
                    bufferEcriture <<= 1;
                    P0.apprendre(0);
                    _borneInf = pivot + 1;
                }

                while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
                {
                    _borneInf <<= 8;
                    _borneSup = (_borneSup << 8) + 0x000000ff;
                    try
                    {
                        buffer = _entree->lireOctet();
                    }
                    catch (EFinFichier &)
                    {
                        buffer  = 0;
                    }
                    reel = (reel << 8) + buffer;
                }
            }

            // composante 1
            for (int i = 0; i < 5; ++i)
            {
                pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P1.p());
                if ( reel <= pivot)
                {
                    bufferEcriture <<= 1;
                    bufferEcriture += 1;
                    P1.apprendre(1);
                    _borneSup = pivot;
                }
                else
                {
                    bufferEcriture <<= 1;
                    P1.apprendre(0);
                    _borneInf = pivot + 1;
                }

                while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
                {
                    _borneInf <<= 8;
                    _borneSup = (_borneSup << 8) + 0x000000ff;
                    try
                    {
                        buffer = _entree->lireOctet();
                    }
                    catch (EFinFichier &)
                    {
                        buffer  = 0;
                    }
                    reel = (reel << 8) + buffer;
                }
            }

            // composante 2
            for (int i = 0; i < 5; ++i)
            {
                pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P2.p());
                if ( reel <= pivot)
                {
                    bufferEcriture <<= 1;
                    bufferEcriture += 1;
                    P2.apprendre(1);
                    _borneSup = pivot;
                }
                else
                {
                    bufferEcriture <<= 1;
                    P2.apprendre(0);
                    _borneInf = pivot + 1;
                }

                while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
                {
                    _borneInf <<= 8;
                    _borneSup = (_borneSup << 8) + 0x000000ff;
                    try
                    {
                        buffer = _entree->lireOctet();
                    }
                    catch (EFinFichier &)
                    {
                        buffer  = 0;
                    }
                    reel = (reel << 8) + buffer;
                }
            }

            _sortie->ecrireOctets(&bufferEcriture, sizeof(U16));
        }

        U8 sign[] =
            {
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0x52,0x55,0x45,0x56,0x49,0x53,0x49,0x4F,0x4E,0x2D,0x58,0x46,0x49,0x4C,0x45,0x2E,0x00
            };
        _sortie->ecrireOctets(sign, 26);

    }
#ifndef SFX
    ////////////////////////////////////////////////////////////////////////////
    void ArTGA::ArTGAgray16Compr(int largeurImage, int hauteurImage)
    {
        U8  buffer;

        //init les variables de parcours du fichier
        int composante = 0;

        while (composante < (largeurImage * hauteurImage * 2))
        {
            try
            {
                buffer = _entree->lireOctet();
            }
            catch (EFinFichier &)
			{
				break;
			}

			switch (composante % 2)
			{
				case 0:
					for ( int i = 7; i >= 0; --i)
					{
						if ((buffer >> i) & 1) //si le bit courant est un 1
						{
							_borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P0.p());
							P0.apprendre(1);
						}
						else
						{
							_borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P0.p()) + 1;
							P0.apprendre(0);
						}
						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_sortie->ecrireOctet((U8)(_borneInf >> 24));
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
						}
					}
					break;
				case 1:
					for ( int i = 7; i >= 0; --i)
					{
						if ((buffer >> i) & 1) //si le bit courant est un 1
						{
							_borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P1.p());
							P1.apprendre(1);
						}
						else
						{
							_borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P1.p()) + 1;
							P1.apprendre(0);
						}
						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_sortie->ecrireOctet((U8)(_borneInf >> 24));
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
						}
					}
					break;
            }
            ++composante;
        }

        _sortie->ecrireOctet((U8)(_borneSup  >> 24));
    }
#endif
    void ArTGA::ArTGAgray16Decompr(int largeurImage, int hauteurImage)
    {
        U16  buffer;
        U32  pivot,
        reel;
        int  i;

        i     = 0;
        reel   = 0;
        buffer = 0;

        while (i < 4)
        {
            try
            {
                buffer = _entree->lireOctet();
            }
            catch (EFinFichier &)
            {
                buffer = 0;
            }
            reel = (reel << 8) + buffer;
            buffer = 0;
            ++i;
        }

        int composante = 0;

        while (composante < (largeurImage * hauteurImage * 2))
        {
            switch (composante % 2)
            {
                case 0:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P0.p());
						if ( reel <= pivot)
						{
							_sortie->ecrireBit(1);
							P0.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							_sortie->ecrireBit(0);
							P0.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch(EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
                case 1:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P1.p());
						if ( reel <= pivot)
						{
							_sortie->ecrireBit(1);
							P1.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							_sortie->ecrireBit(0);
							P1.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch(EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
            }
            ++composante;
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    void ArTGA::ArTGA24RLECompr(int largeurImage, int hauteurImage)
    {
        U8  buffer;

        //init les variables de parcours du fichier
        int composante = 0;
        int size = 0;
        bool rle = true;
        U32 bigBuffer = 0;

        while (composante < (largeurImage * hauteurImage * 3))
        {
            if (size == 0)
            {
                try
                {
                    U8 tmp;
                    tmp = _entree->lireOctet();
                    //printf("octet de compression: %0x\n", tmp);
                    size = (tmp & 0x7f) + 1;
                    //printf("size: %d\n", size);
                    if (tmp & 0x80) // si le bloc est compresser
                    {
                        rle = true;
                        bigBuffer = _entree->lireOctet();
                        buffer = (U8)(bigBuffer & 0xff);
                        bigBuffer <<= 8;
                        bigBuffer += _entree->lireOctet();
                        bigBuffer <<= 8;
                        bigBuffer += _entree->lireOctet();
                        //printf("bigBuffer: %0x\n", bigBuffer);
                    }
                    else
                    {
                        rle = false;
                        buffer = _entree->lireOctet();
                    }
                }
                catch (EFinFichier &)
                {
                    printf("pas cool 1\n");
                    break;
                }
            }
            else
            {
                if (rle)
                {
                    buffer = (U8)((bigBuffer >> (16 - (composante  % 3) * 8)) & 0xff);
                }
                else
                {
                    try
                    {
                        buffer = _entree->lireOctet();
                    }
                    catch (EFinFichier &)
                    {
                        printf("pas cool 2\n");
                        break;
                    }
                }
            }

            //printf("buffer: %0x\n", buffer);

            switch (composante % 3)
            {
                case 0:
					for ( int i = 7; i >= 0; --i)
					{
						if ((buffer >> i) & 1) //si le bit courant est un 1
						{
							_borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P0.p());
							P0.apprendre(1);
						}
						else
						{
							_borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P0.p()) + 1;
							P0.apprendre(0);
						}
						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_sortie->ecrireOctet((U8)(_borneInf >> 24));
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
						}
					}
					break;
                case 1:
					for ( int i = 7; i >= 0; --i)
					{
						if ((buffer >> i) & 1) //si le bit courant est un 1
						{
							_borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P1.p());
							P1.apprendre(1);
						}
						else
						{
							_borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P1.p()) + 1;
							P1.apprendre(0);
						}
						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_sortie->ecrireOctet((U8)(_borneInf >> 24));
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
						}
					}
					break;
                case 2:
					for ( int i = 7; i >= 0; --i)
					{
						if ((buffer >> i) & 1) //si le bit courant est un 1
						{
							_borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P2.p());
							P2.apprendre(1);
						}
						else
						{
							_borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P2.p()) + 1;
							P2.apprendre(0);
						}
						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_sortie->ecrireOctet((U8)(_borneInf >> 24));
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
						}
					}
					break;
            }
            ++composante;
            if (!(composante % 3))
            {
                //printf("size: %d\n", size);
                --size; // on a coder un pixel
            }
        }

        while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
        {
            _sortie->ecrireOctet((U8)(_borneInf >> 24));
            _borneInf <<= 8;
            _borneSup = (_borneSup << 8) + 0x000000ff;
        }

        _sortie->ecrireOctet((U8)(_borneSup  >> 24));
    }

    void ArTGA::ArTGA24RLEDecompr(int largeurImage, int hauteurImage)
    {
        U16  buffer;
        U32  pivot,
        reel;
        int  i;
        Tab<U32> bloc(largeurImage); // bloc ou l'on va mettre les pixels decompresser a recompresser en rle

        i     = 0;
        reel   = 0;
        buffer = 0;

        while (i < 4)
        {
            try
            {
                buffer = _entree->lireOctet();
            }
            catch (EFinFichier &)
            {
                buffer = 0;
            }
            reel = (reel << 8) + buffer;
            buffer = 0;
            ++i;
        }

        int composante = 0;
        U32 bigBuffer = 0;

        while (composante < (largeurImage * hauteurImage * 3))
        {
            switch (composante % 3)
            {
                case 0:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P0.p());
						if ( reel <= pivot)
						{
							//_sortie->ecrireBit(1);
							bigBuffer += 1;
							bigBuffer <<= 1;
							P0.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							//_sortie->ecrireBit(0);
							bigBuffer <<= 1;
							P0.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch(EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
                case 1:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P1.p());
						if ( reel <= pivot)
						{
							//_sortie->ecrireBit(1);
							bigBuffer += 1;
							bigBuffer <<= 1;
							P1.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							//_sortie->ecrireBit(0);
							bigBuffer <<= 1;
							P1.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch(EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
                case 2:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P2.p());
						if ( reel <= pivot)
						{
							//_sortie->ecrireBit(1);
							bigBuffer += 1;
							bigBuffer <<= 1;
							P2.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							//_sortie->ecrireBit(0);
							bigBuffer <<= 1;
							P2.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch(EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
            }

			++composante;
			if (!(composante % 3)) // un pixel est fini de decompresser
			{
				bloc[(composante / 3 - 1) % largeurImage] = (bigBuffer >> 1) & 0xffffff;
				bigBuffer = 0;
				if (((composante / 3 - 1) % largeurImage) == (largeurImage - 1))
				{
					/*compresser le bloc[] car il est plein et l'ecrie dans la sortie*/
					Rle24(bloc, largeurImage);
				}
			}
        }

        U8 sign[] =
            {
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0x52,0x55,0x45,0x56,0x49,0x53,0x49,0x4F,0x4E,0x2D,0x58,0x46,0x49,0x4C,0x45,0x2E,0x00
            };
        _sortie->ecrireOctets(sign, 26);
    }


    void ArTGA::Rle24(Tab<U32> &bloc, int largeurImage)
    {
        Tab<U32> tabRAW(largeurImage);
        int i = 0, j = 0;
        int posRAW = 0;

        while (i < largeurImage)
        {
            j = 0;

            while (((i + j) < largeurImage) && (bloc[i] == bloc[i + j]) && (j < 128))
            {
                ++j;
            }

            if (j >= 2)
            {
                if (posRAW)
                {
                    _sortie->ecrireOctet((U8)((posRAW - 1) & 0x7f));
                    for (int a = 0; a < posRAW; ++a)
                    {
                        _sortie->ecrireOctet((U8)(tabRAW[a] >> 16));
                        _sortie->ecrireOctet((U8)(tabRAW[a] >> 8));
                        _sortie->ecrireOctet((U8)(tabRAW[a]));
                    }
                    posRAW = 0;
                }
                _sortie->ecrireOctet((U8)((0x80 + j - 1) & 0xff));
                _sortie->ecrireOctet((U8)(bloc[i] >> 16));
                _sortie->ecrireOctet((U8)(bloc[i] >> 8));
                _sortie->ecrireOctet((U8)(bloc[i]));
            }
            else
            {
            	// truc bizarre de la compression rle: si les pixels sont
            	// comme les pixels rle precedent on fait comme si c'etait un bloc rle. (0x80 + j - 1)

                for (int l = 0; l < j; ++l)
                {
                    tabRAW[posRAW++] = bloc[i + l];
                    if ((posRAW == largeurImage) || (posRAW >= 128))
                    {
                        _sortie->ecrireOctet((U8)((posRAW - 1) & 0x7f));
                        for (int a = 0; a < posRAW; ++a)
                        {
                            _sortie->ecrireOctet((U8)(tabRAW[a] >> 16));
                            _sortie->ecrireOctet((U8)(tabRAW[a] >> 8));
                            _sortie->ecrireOctet((U8)(tabRAW[a]));
                        }
                        posRAW = 0;
                    }
                }
            }
            i = i + j;
        }

        if (posRAW)
        {
            _sortie->ecrireOctet((U8)((posRAW - 1) & 0x7f));
            for (int a = 0; a < posRAW; ++a)
            {
                _sortie->ecrireOctet((U8)(tabRAW[a] >> 16));
                _sortie->ecrireOctet((U8)(tabRAW[a] >> 8));
                _sortie->ecrireOctet((U8)(tabRAW[a]));
            }
        }
    }

    ////////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////////
    void ArTGA::ArTGA32RLECompr(int largeurImage, int hauteurImage)
    {
        U8  buffer;

        //init les variables de parcours du fichier
        int composante = 0;
        int size = 0;
        bool rle = true;
        U32 bigBuffer = 0;

        while (composante < (largeurImage * hauteurImage * 4))
        {
            if (size == 0) //charge un nouveau bloc si le bloc precedent est fini et init le buffer pour le premier passage
            {
                try
                {
                    U8 tmp;
                    tmp = _entree->lireOctet();
                    //printf("octet de compression: %0x\n", tmp);
                    size = (tmp & 0x7f) + 1;
                    //printf("size: %d\n", size);
                    if (tmp & 0x80) // si le bloc est compresse
                    {
                        rle = true;
                        bigBuffer = _entree->lireOctet();
                        buffer = (U8)(bigBuffer & 0xff);
                        bigBuffer <<= 8;
                        bigBuffer |= _entree->lireOctet();
                        bigBuffer <<= 8;
                        bigBuffer |= _entree->lireOctet();
                        bigBuffer <<= 8;
                        bigBuffer |= _entree->lireOctet();
                        //printf("bigBuffer: %0x\n", bigBuffer);
                    }
                    else
                    {
                        rle = false;
                        buffer = _entree->lireOctet();
                    }
                }
                catch (EFinFichier &)
                {
                    printf("pas cool 1\n");
                    break;
                }
            }
            else
            {
                if (rle) // si le bloc est compresser, on utilise le biBuffer pour init le buffer a encoder
                {
                    buffer = (U8)((bigBuffer >> (24 - (composante  % 4) * 8)) & 0xff);
                }
                else // sinon on lit dans le fichier le buffer a encoder
                {
                    try
                    {
                        buffer = _entree->lireOctet();
                    }
                    catch (EFinFichier &)
                    {
                        printf("pas cool 2\n");
                        break;
                    }
                }
            }

            //printf("buffer: %0x\n", buffer);

            switch (composante % 4)
            {
                case 0:
					for ( int i = 7; i >= 0; --i)
					{
						if ((buffer >> i) & 1) //si le bit courant est un 1
						{
							_borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P0.p());
							P0.apprendre(1);
						}
						else
						{
							_borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P0.p()) + 1;
							P0.apprendre(0);
						}
						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_sortie->ecrireOctet((U8)(_borneInf >> 24));
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
						}
					}
					break;
                case 1:
					for ( int i = 7; i >= 0; --i)
					{
						if ((buffer >> i) & 1) //si le bit courant est un 1
						{
							_borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P1.p());
							P1.apprendre(1);
						}
						else
						{
							_borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P1.p()) + 1;
							P1.apprendre(0);
						}
						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_sortie->ecrireOctet((U8)(_borneInf >> 24));
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
						}
					}
					break;
                case 2:
					for ( int i = 7; i >= 0; --i)
					{
						if ((buffer >> i) & 1) //si le bit courant est un 1
						{
							_borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P2.p());
							P2.apprendre(1);
						}
						else
						{
							_borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P2.p()) + 1;
							P2.apprendre(0);
						}
						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_sortie->ecrireOctet((U8)(_borneInf >> 24));
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
						}
					}
					break;
				case 3:
					for ( int i = 7; i >= 0; --i)
					{
						if ((buffer >> i) & 1) //si le bit courant est un 1
						{
							_borneSup = _borneInf + (((_borneSup - _borneInf) >> 12) * P3.p());
							P3.apprendre(1);
						}
						else
						{
							_borneInf = _borneInf + (((_borneSup - _borneInf) >> 12)  * P3.p()) + 1;
							P3.apprendre(0);
						}
						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_sortie->ecrireOctet((U8)(_borneInf >> 24));
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
						}
					}
					break;
            }

            ++composante;
            if (!(composante % 4))
            {
                //printf("size: %d\n", size);
                --size; // on a code un pixel
            }
        }

        _sortie->ecrireOctet((U8)(_borneSup  >> 24));
    }

    void ArTGA::ArTGA32RLEDecompr(int largeurImage, int hauteurImage)
    {
        U16  buffer;
        U32  pivot,
        reel;
        int  i;
        Tab<U32> bloc(largeurImage); // bloc ou l'on va mettre les pixels decompresser a recompresser en rle

        i     = 0;
        reel   = 0;
        buffer = 0;

        while (i < 4)
        {
            try
            {
                buffer = _entree->lireOctet();
            }
            catch (EFinFichier &)
            {
                buffer = 0;
            }
            reel = (reel << 8) + buffer;
            buffer = 0;
            ++i;
        }

        int composante = 0;
        U32 bigBuffer = 0;

        while (composante < (largeurImage * hauteurImage * 4))
        {
            switch (composante % 4)
            {
                case 0:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P0.p());
						bigBuffer <<= 1;
						if ( reel <= pivot)
						{
							//_sortie->ecrireBit(1);
							bigBuffer += 1;
							P0.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							//_sortie->ecrireBit(0);
							P0.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch (EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
                case 1:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P1.p());
						bigBuffer <<= 1;
						if ( reel <= pivot)
						{
							//_sortie->ecrireBit(1);
							bigBuffer += 1;
							P1.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							//_sortie->ecrireBit(0);
							P1.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch (EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
                case 2:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P2.p());
						bigBuffer <<= 1;
						if ( reel <= pivot)
						{
							//_sortie->ecrireBit(1);
							bigBuffer += 1;
							P2.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							//_sortie->ecrireBit(0);
							P2.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch (EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
				case 3:
					for (uint i = 0; i < 8; ++i)
					{
						pivot = _borneInf + (((_borneSup - _borneInf) >> 12) * P3.p());
						bigBuffer <<= 1;
						if ( reel <= pivot)
						{
							//_sortie->ecrireBit(1);
							bigBuffer += 1;
							P3.apprendre(1);
							_borneSup = pivot;
						}
						else
						{
							//_sortie->ecrireBit(0);
							P3.apprendre(0);
							_borneInf = pivot + 1;
						}

						while (((_borneInf ^ _borneSup) & 0xff000000) == 0)
						{
							_borneInf <<= 8;
							_borneSup = (_borneSup << 8) + 0x000000ff;
							try
							{
								buffer = _entree->lireOctet();
							}
							catch (EFinFichier &)
							{
								buffer  = 0;
							}
							reel = (reel << 8) + buffer;
						}
					}
					break;
            }

			++composante;
			if (!(composante % 4)) // un pixel est fini de decompresser
			{
				bloc[(composante / 4 - 1) % largeurImage] = bigBuffer & 0xffffffff;
				bigBuffer = 0;

				if (((composante / 4 - 1) % largeurImage) == (largeurImage - 1))
				{
					Rle32(bloc, largeurImage);
				}
			}
        }

        U8 sign[] =
            {
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0x52,0x55,0x45,0x56,0x49,0x53,0x49,0x4F,0x4E,0x2D,0x58,0x46,0x49,0x4C,0x45,0x2E,0x00
            };
        _sortie->ecrireOctets(sign, 26);
    }


    void ArTGA::Rle32(Tab<U32> &bloc, int largeurImage)
    {
        Tab<U32> tabRAW(largeurImage);
        int i = 0, j = 0;
        int posRAW = 0;

        while (i < largeurImage)
        {
            j = 1;

            while (((i + j) < largeurImage) && (bloc[i] == bloc[i + j]) && (j < 128))
            {
                ++j;
            }

            if (j >= 2)
            {
                if (posRAW)
                {
                    _sortie->ecrireOctet((U8)((posRAW - 1) & 0x7f));
                    for (int a = 0; a < posRAW; ++a)
                    {
                    	_sortie->ecrireOctet((U8)(tabRAW[a] >> 24));
                    	_sortie->ecrireOctet((U8)(tabRAW[a] >> 16));
                    	_sortie->ecrireOctet((U8)(tabRAW[a] >> 8));
                        _sortie->ecrireOctet((U8)(tabRAW[a]));
                    }
                    posRAW = 0;
                }
                _sortie->ecrireOctet((U8)((0x80 + j - 1) & 0xff));
                _sortie->ecrireOctet((U8)(bloc[i] >> 24));
                _sortie->ecrireOctet((U8)(bloc[i] >> 16));
                _sortie->ecrireOctet((U8)(bloc[i] >> 8));
                _sortie->ecrireOctet((U8)(bloc[i]));
            }
            else
            {
            	//j = 1
				tabRAW[posRAW++] = bloc[i];
				if ((posRAW == largeurImage) || (posRAW >= 128))
				{
					_sortie->ecrireOctet((U8)((posRAW - 1) & 0x7f));
					for (int a = 0; a < posRAW; ++a)
					{
						_sortie->ecrireOctet((U8)(tabRAW[a] >> 24));
						_sortie->ecrireOctet((U8)(tabRAW[a] >> 16));
						_sortie->ecrireOctet((U8)(tabRAW[a] >> 8));
						_sortie->ecrireOctet((U8)(tabRAW[a]));
					}
					posRAW = 0;
				}
            }
            i = i + j;
        }

        if (posRAW)
        {
            _sortie->ecrireOctet((U8)((posRAW - 1) & 0x7f));
            for (int a = 0; a < posRAW; ++a)
            {
            	_sortie->ecrireOctet((U8)(tabRAW[a] >> 24));
                _sortie->ecrireOctet((U8)(tabRAW[a] >> 16));
                _sortie->ecrireOctet((U8)(tabRAW[a] >> 8));
                _sortie->ecrireOctet((U8)(tabRAW[a]));
            }
        }
    }

    ////////////////////////////////////////////////////////////////////////////
}


