/*
** Big Crunch
** Compresseur exploitable et exprimental
**
** Version ligne de commande
**
** Copyright (C) 2005-2006 - KaBoom
**
** http://bcrunch.online.fr/
*/

/*
** Ce fichier ne doit pas tre inclus  la compilation !
*/

#include"Commande.hpp"


namespace BigCrunch
{

	void Commande::executer(int nbMots, char *commande[])
	{
		ParametresCommande parametres;

		if (!Parseur::parser(nbMots, commande, parametres))
		{
			return;
		}

#ifndef SFX

		if (parametres.nomSortie == "")
		{
			if (parametres.plateForme == NON)
			{
				if (!Fichier::Existe("archive.bc"))
				{
					parametres.nomSortie = "archive.bc";
				}
				else
				{
					parametres.nomSortie = Fichier::Nommer("archive.bc");
				}
			}
			else
			{
				if (!Fichier::Existe("archive.exe"))
				{
					parametres.nomSortie = "archive.exe";
				}
				else
				{
					parametres.nomSortie = Fichier::Nommer("archive.exe");
				}
			}
		}

		if (parametres.logger)
		{
			if (parametres.log == "")
			{
				parametres.log = Fichier::changerExtension(parametres.nomSortie.c_str(), ".log");
			}

			parametres.logf = fopen(parametres.log.c_str(), "w");
			printl("+----------------+");
			printl("|Log de BigCrunch|");
			printl("+----------------+");
			printl(parametres.nomSortie.c_str());
			printl("");
		}

		if (parametres.fichiers->EstVide())
		{
			printf("Aucun fichier specifie en entree\n");
			printl("Aucun fichier specifie en entree\n");

			return;
		}

		switch (parametres.action)
		{
			case AUCUNE:
				printf("Aucune action specifiee\n");
				return;
			case AJOUTER:
				ajouter(parametres);
				break;
			case EXTRAIRE:
#endif
				extraire(parametres);
#ifndef SFX
				break;
			case SUPPRIMER:
				supprimer(parametres);
				break;
			case LISTER:
				lister(parametres);
				break;
			case CONVERTIR:
				convertir(parametres);
				break;
			case TESTER:
				tester(parametres);
				break;
		}

		if (parametres.logger)
		{
			fclose(parametres.logf);
		}

	}

	void Commande::ajouter(ParametresCommande &parametres)
	{
		Fichier sortie(parametres.nomSortie.c_str(), ECRITURE);
		FluxBinaire *fluxSortie = NULL;
		fluxSortie = sortie.GetFlux();

		if (!parametres.miseAJour)
		{

			if (parametres.plateForme != NON)
			{
				switch (parametres.plateForme)
				{
					case NON:
						break;
					case WIN:
						sortie.copier("Win32_x86.sfx");
						break;
					case LINUX:
						sortie.copier("Linux_x86.sfx");
						break;
					case BSD:
						sortie.copier("BSD_x86.sfx");
						break;
					case MACOS:
						sortie.copier("MacOS_x86.sfx");
						break;
				}
			}

			fluxSortie->ecrireOctet(ENTETE_BCRUNCH);
			fluxSortie->ecrireOctet(VERSION_BCRUNCH);

		}

		// On initialise la taille totale des fichiers non compresss
		uint tailleTotaleDecompresse = 0;
		uint tailleTotaleCompresse   = 0;
		uint nbFichiers              = 0;

		// On initialise le temps du dbut de la compression
		clock_t tempsDepartTotal = 0;
		tempsDepartTotal = clock();


		// Compression des fichiers

		for (uint type = 0; type < NB_TYPES; ++type)
		{
			while (!parametres.tabTypes[type].empty())
			{
				// On rcupre le nom du fichier
				string nomFichier = parametres.tabTypes[type].front();
				parametres.tabTypes[type].pop();

				// On garde sa taille dcompress
				uint tailleDecompresse = Fichier::Taille(nomFichier.c_str());

				// On affiche ce que l'on fait
				printf("Compression et ajout de %s...\n", nomFichier.c_str());

				clock_t tempsDepart = clock();

				try
				{
					Codeur *codeur = NULL;
					//Appel du codeur en fonction du type de fichier
					switch (type)
					{
						case TXT :
							codeur = new ArSa1(nomFichier.c_str(), COMPRESSER);
							printf("Algorithme: ArSa1                          0.0%%");
							break;
						case EXE :
							/*codeur = new ArBin(nomFichier.c_str(), COMPRESSER);
							printf("Algorithme: ArBin                          0.0%%");
							break;*/
						case UKN :
							codeur = new ArFaMix(nomFichier.c_str(), COMPRESSER);
							printf("Algorithme: ArFaMix                        0.0%%");
							break;
						case BMP :
							codeur = new ArBMP(nomFichier.c_str(), COMPRESSER);
							printf("Algorithme: ArBMP                          0.0%%");
							break;
						case TGA :
							codeur = new ArTGA(nomFichier.c_str(), COMPRESSER);
							printf("Algorithme: ArTGA                          0.0%%");
							break;
						case INC :
							codeur = new Copie(nomFichier.c_str(), COMPRESSER);
							printf("Copie du fichier source (incompressible)   0.0%%");
							break;
						default :
							printf("Erreur endroit non logique\n");
					}

					const char *nomTmp = codeur->compresser();
					delete codeur;

					// On calcule sa taille compress
					uint tailleCompresse = Fichier::Taille(nomTmp);

					// Si il a grossi, on le met dans les fichiers incompressibles
					if ((type != INC) && (tailleDecompresse <= tailleCompresse))
					{
						parametres.tabTypes[INC].push(nomFichier);
						Fichier::supprimer(nomTmp);
						continue;
					}

					// On crit le nom du fichier dans l'archive
					const char *pNomFichier = nomFichier.c_str();
					for (uint i = 0; pNomFichier[i] != '\0'; ++i)
					{
						fluxSortie->ecrireOctet(pNomFichier[i]);
					}

					// On prcise qu'il s'agit bien de texte
					fluxSortie->ecrireOctet((U8)(type & 0xff));

					// On crit la taille du fichier compress
					fluxSortie->ecrireOctets((void *)&tailleCompresse, sizeof(uint));

					// On recopie le fichier compress dans l'archive
					sortie.copier(nomTmp);

					// On supprime le fichier temporaire
					Fichier::supprimer(nomTmp);

					// On affiche les rsultats
					if (parametres.details >= 3)
					{
						float ratio;
						if (tailleDecompresse != 0)
						{
							ratio = (float)tailleCompresse / tailleDecompresse;
						}
						else
						{
							ratio = 0;
						}
						printf("\b\b\b\b\b\b      \n\t%i octets -> %i octets (%.2f%%, %.3f bpc) en %i seconde(s)\n",
							   tailleDecompresse, tailleCompresse,
							   ratio * 100, ratio * 8, (int)((clock() - tempsDepart) / 1000));
					}
					else
					{
						printf("\b\b\b\b\b\b100.0%%\n");
					}

					// On incrmente le nombre de fichiers
					++nbFichiers;
					tailleTotaleDecompresse += tailleDecompresse;
					tailleTotaleCompresse   += tailleCompresse;
				}
				catch (EFichierMauvais &)
				{
					// Si ce n'tait pas le bon type, on le met dans les fichiers inconnus
					parametres.tabTypes[UKN].push(nomFichier);
				}

				// On supprime le fichier d'origine si a a t demand
				if (parametres.effacer)
				{
					Fichier::supprimer(nomFichier.c_str());
				}

				// On crit le CRC32 (4 octets)
				// TODO calculer le CRC32 avant pour la recherche de doublons
				fluxSortie->ecrireU32(Fichier::Crc32(nomFichier.c_str()));

			}

		}

		if(parametres.cloche)
		{
			printf("\a");
		}

		if (parametres.details >= 2)
		{
			float ratio;

			if (tailleTotaleDecompresse != 0)
			{
				ratio = (float)tailleTotaleCompresse / tailleTotaleDecompresse;
			}
			else
			{
				ratio = 0;
			}
			printf("\n%i fichier(s) compresse(s)\n\t%i octets -> %i octets (%.2f%%, %.3f bpc) en %i seconde(s)\n",
				   nbFichiers, tailleTotaleDecompresse, tailleTotaleCompresse,
				   ratio * 100, ratio * 8, (int)((clock() - tempsDepartTotal) / 1000));
		}

#endif
	}


	void Commande::extraire(ParametresCommande &parametres)
	{
		FluxBinaire *fluxEntree = new FluxBinaire(parametres.tabTypes[0].front().c_str(), LECTURE);

		if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
		{
			fluxEntree->aller(TAILLE_EXE_WIN32);

			if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
			{
				fluxEntree->aller(TAILLE_EXE_LINUX);

				if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
				{
					fluxEntree->aller(TAILLE_EXE_BSD);

					if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
					{
						fluxEntree->aller(TAILLE_EXE_MACOS);

						if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
						{
							printf("Format d'archive incompatible avec Big Crunch, impossible de decompresser\n");
							throw ENTETE_BCRUNCH;
						}
					}
				}
			}
		}

		byte version = fluxEntree->lireOctet();

		if (version != VERSION_BCRUNCH)
		{
			if (version < VERSION_BCRUNCH)
			{
				printf("Cette archive a ete compressee avec une ancienne version de BigCrunch (%ui), impossible de decompresser\n", version);
			}
			else
			{
				printf("Cette archive a ete compressee avec une nouvelle version de BigCrunch (%ui), impossible de decompresser\n", version);
			}
			throw VERSION_BCRUNCH;
		}

		queue<string> listeFichier;
		queue<string> listeFichier2;

		forever
		{

			try
			{

				string nomFichier = "";
				U8 caractere;

				while ((caractere = fluxEntree->lireOctet()) > 31)
				{
					nomFichier += caractere;
				}

				listeFichier.push(nomFichier);

				if (caractere == ID)
				{
					continue;
				}

				while (!listeFichier.empty())
				{
					nomFichier = listeFichier.front();
					listeFichier.pop();

					// Si le fichier qu'on veut extraire existe dj
					if (Fichier::Existe(nomFichier.c_str()))
					{
						switch (parametres.reaction)
						{
							case PASSER:
								printf("Fichier deja existant %s...\n", nomFichier.c_str());
								continue;
							case RENOMMER:
								nomFichier = Fichier::Nommer(nomFichier.c_str());
								break;
							case ECRASER:
								//printf("yeah\n");
								Fichier::supprimer(nomFichier.c_str());
								break;
						}
					}

					printf("Extraction de %s...\n", nomFichier.c_str());
					listeFichier2.push(nomFichier);
				}

				if (!listeFichier2.empty())
				{
					nomFichier = listeFichier2.front();
					listeFichier2.pop();

					uint tailleFichierCompresse = *((uint *)fluxEntree->lireOctets(sizeof(uint)));

					char *nomTmp = Fichier::NomFichierTemporaire();

					FluxBinaire *fluxFichierTemporaire = new FluxBinaire(nomTmp, ECRITURE);

					for (uint i = 0; i < tailleFichierCompresse; ++i)
					{
						U8 tampon = fluxEntree->lireOctet();
						fluxFichierTemporaire->ecrireOctet(tampon);
					}

					delete fluxFichierTemporaire;

					Codeur *codeur = NULL;
					//Appel du codeur en fonction du type de fichier
					switch (caractere)
					{
						case TXT :
							codeur = new ArSa1(nomTmp, DECOMPRESSER, nomFichier.c_str());
							printf("Algorithme: ArSa1                          0.0%%");
							break;
						case EXE :
							/*codeur = new ArBin(nomTmp, DECOMPRESSER, nomFichier.c_str());
							printf("Algorithme: ArBin                          0.0%%");
							break;*/
						case UKN :
							codeur = new ArFaMix(nomTmp, DECOMPRESSER, nomFichier.c_str());
							printf("Algorithme: ArFaMix                        0.0%%");
							break;
						case BMP :
							codeur = new ArBMP(nomTmp, DECOMPRESSER, nomFichier.c_str());
							printf("Algorithme: ArBMP                          0.0%%");
							break;
						case TGA :
							codeur = new ArTGA(nomTmp, DECOMPRESSER, nomFichier.c_str());
							printf("Algorithme: ArTGA                          0.0%%");
							break;
						case INC :
							codeur = new Copie(nomTmp, DECOMPRESSER, nomFichier.c_str());
							printf("Copie du fichier source (incompressible)   0.0%%");
							break;
						default :
							printf("Erreur endroit non logique\n");
					}
					codeur->decompresser();
					delete codeur;



					// On supprime le fichier temporaire
					Fichier::supprimer(nomTmp);

					printf("\b\b\b\b\b\b100.0%%\n");

					// TODO Jems : comparer les CRC32
					if (fluxEntree->lireU32() != Fichier::Crc32(nomFichier.c_str()))
					{
						printf("Erreur de controle de redondance cyclique\n");
						printf("Le fichier %s est corrompu\n", nomFichier.c_str());

						if (!parametres.forcer)
						{
							Fichier::supprimer(nomFichier.c_str());
						}
					}

					while (!listeFichier2.empty())
					{
						string nomCopie = listeFichier2.front();
						listeFichier2.pop();

						Fichier::copier(nomFichier.c_str(), nomCopie.c_str());
					}
				}
				else
				{
					uint tailleFichierCompresse = *((uint *)fluxEntree->lireOctets(sizeof(uint)));

					// Les 4 octets supplmentaires sont pour le CRC32
					fluxEntree->avancer(tailleFichierCompresse + 4);
				}
			}
			catch (EFinFichier &)
			{
				break;
			}

		}

		if(parametres.cloche)
		{
			printf("\a");
		}
	}

#ifndef SFX

	void Commande::supprimer(ParametresCommande &parametres)
	{
		char *nomArchive = new char[strlen(parametres.tabTypes[0].front().c_str())];
		strcpy(nomArchive, parametres.tabTypes[0].front().c_str());

		FluxBinaire *fluxEntree = NULL;
		fluxEntree = new FluxBinaire(nomArchive, LECTURE);

		parametres.tabTypes[0].pop();

		if (parametres.tabTypes[0].empty())
		{
			printf("Aucun fichier a supprimer passe en parametre\n");
			return;
		}

		if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
		{
			fluxEntree->aller(TAILLE_EXE_WIN32);

			if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
			{
				fluxEntree->aller(TAILLE_EXE_LINUX);

				if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
				{
					fluxEntree->aller(TAILLE_EXE_BSD);

					if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
					{
						fluxEntree->aller(TAILLE_EXE_MACOS);

						if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
						{
							printf("Format d'archive incompatible avec Big Crunch, impossible de supprimer\n");
							throw ENTETE_BCRUNCH;
						}
					}
				}
			}
		}

		byte version = fluxEntree->lireOctet();

		if (version != VERSION_BCRUNCH)
		{
			if (version < VERSION_BCRUNCH)
			{
				printf("Cette archive a ete compressee avec une ancienne version de BigCrunch (%ui), impossible de supprimer\n", version);
			}
			else
			{
				printf("Cette archive a ete compressee avec une nouvelle version de BigCrunch (%ui), impossible de supprimer\n", version);
			}
			throw VERSION_BCRUNCH;
		}

		char *nomTmp = NULL;
		nomTmp = Fichier::NomFichierTemporaire();

		FluxBinaire *fluxFichierTemporaire = NULL;
		fluxFichierTemporaire = new FluxBinaire(nomTmp, ECRITURE);

		fluxFichierTemporaire->ecrireOctet(ENTETE_BCRUNCH);
		fluxFichierTemporaire->ecrireOctet(VERSION_BCRUNCH);

		U8 caractere = UKN;
		U8 typePrecedent = UKN;

		forever
		{

			try
			{

				string nomFichier = "";

				typePrecedent = caractere;

				while ((caractere = fluxEntree->lireOctet()) > 31)
				{
					nomFichier += caractere;
				}


				if (parametres.fichiers->contientEtSupprime(nomFichier))
				{
					if ((caractere != ID) && (typePrecedent != ID))
					{
						uint tailleFichierCompresse = *((uint *)fluxEntree->lireOctets(sizeof(uint)));

						// Les 4 octets supplmentaires sont pour le CRC32
						fluxEntree->avancer(tailleFichierCompresse + 4);
					}

					if (typePrecedent == ID)
					{
						fluxFichierTemporaire->ecrireOctet(caractere);
					}

					caractere = UKN;

					printf("Suppression de %s...\n", nomFichier.c_str());
				}
				else
				{
					if (typePrecedent == ID)
					{
						fluxFichierTemporaire->ecrireOctet(typePrecedent);
					}

					// On crit le nom du fichier dans l'archive
					const char *pNomFichier = nomFichier.c_str();
					for (uint i = 0; pNomFichier[i] != '\0'; ++i)
					{
						fluxFichierTemporaire->ecrireOctet(pNomFichier[i]);
					}

					if (caractere != ID)
					{
						// On prcise le type du fichier
						fluxFichierTemporaire->ecrireOctet(caractere);

						uint tailleFichierCompresse = *((uint *)fluxEntree->lireOctets(sizeof(uint)));


						// On crit la taille du fichier compress
						fluxFichierTemporaire->ecrireOctets((void *)&tailleFichierCompresse, sizeof(uint));

						for (uint i = 0; i < tailleFichierCompresse + 4; ++i)
						{
							U8 tampon = fluxEntree->lireOctet();
							fluxFichierTemporaire->ecrireOctet(tampon);
						}
					}
				}

			}
			catch (EFinFichier &)
			{
				break;
			}
		}


		if (!parametres.tabTypes[0].empty())
		{
			printf("Impossible de supprimer les fichiers suivants :\n");
			while (!parametres.tabTypes[0].empty())
			{
				string nomFichier = parametres.tabTypes[0].front();
				parametres.tabTypes[0].pop();
				printf("%s\n", nomFichier.c_str());
			}
		}

		delete fluxEntree;
		delete fluxFichierTemporaire;

		Fichier::supprimer(nomArchive);
		Fichier::renommer(nomTmp, nomArchive);

		delete [] nomArchive;
	}

	void Commande::lister(ParametresCommande &parametres)
	{
		FluxBinaire *fluxEntree = new FluxBinaire(parametres.tabTypes[0].front().c_str(), LECTURE);

		if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
		{
			fluxEntree->aller(TAILLE_EXE_WIN32);

			if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
			{
				fluxEntree->aller(TAILLE_EXE_LINUX);

				if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
				{
					fluxEntree->aller(TAILLE_EXE_BSD);

					if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
					{
						fluxEntree->aller(TAILLE_EXE_MACOS);

						if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
						{
							printf("Format d'archive incompatible avec Big Crunch, impossible de lister\n");
							throw ENTETE_BCRUNCH;
						}
					}
				}
			}
		}

		byte version = fluxEntree->lireOctet();

		if (version != VERSION_BCRUNCH)
		{
			if (version < VERSION_BCRUNCH)
			{
				printf("Cette archive a ete compressee avec une ancienne version de BigCrunch (%ui), impossible de lister\n", version);
			}
			else
			{
				printf("Cette archive a ete compressee avec une nouvelle version de BigCrunch (%ui), impossible de lister\n", version);
			}
			throw VERSION_BCRUNCH;
		}

		forever
		{

			try
			{

				string nomFichier = "";
				U8 caractere;

				while ((caractere = fluxEntree->lireOctet()) > 31)
				{
					nomFichier += caractere;
				}

				printf("%s\n", nomFichier.c_str());

				if (caractere != ID)
				{
					int tailleFichierCompresse = *((int *)fluxEntree->lireOctets(sizeof(uint)));

					// Les 4 octets supplmentaires sont pour le CRC32
					fluxEntree->avancer(tailleFichierCompresse + 4);
				}

			}
			catch (EFinFichier &)
			{
				break;
			}
		}
	}

	void Commande::convertir(ParametresCommande &parametres)
	{
		FluxBinaire *fluxEntree = new FluxBinaire(parametres.tabTypes[0].front().c_str(), LECTURE);

		if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
		{
			fluxEntree->aller(TAILLE_EXE_WIN32);

			if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
			{
				fluxEntree->aller(TAILLE_EXE_LINUX);

				if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
				{
					fluxEntree->aller(TAILLE_EXE_BSD);

					if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
					{
						fluxEntree->aller(TAILLE_EXE_MACOS);

						if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
						{
							printf("Format d'archive incompatible avec Big Crunch, impossible de convertir\n");
							throw ENTETE_BCRUNCH;
						}
					}
				}
			}
		}

		byte version = fluxEntree->lireOctet();

		if (version != VERSION_BCRUNCH)
		{
			if (version < VERSION_BCRUNCH)
			{
				printf("Cette archive a ete compressee avec une ancienne version de BigCrunch (%ui), impossible de convertir\n", version);
			}
			else
			{
				printf("Cette archive a ete compressee avec une nouvelle version de BigCrunch (%ui), impossible de convertir\n", version);
			}
			throw VERSION_BCRUNCH;
		}

		Fichier sortie(parametres.nomSortie.c_str(), ECRITURE);
		FluxBinaire *fluxSortie = sortie.GetFlux();


		if (parametres.plateForme != NON)
		{
			switch (parametres.plateForme)
			{
				case NON:
					break;
				case WIN:
					sortie.copier("Win32_x86.sfx");
					break;
				case LINUX:
					sortie.copier("Linux_x86.sfx");
					break;
				case BSD:
					sortie.copier("BSD_x86.sfx");
					break;
				case MACOS:
					sortie.copier("MacOS_x86.sfx");
					break;
			}
		}

		fluxSortie->ecrireOctet(ENTETE_BCRUNCH);
		fluxSortie->ecrireOctet(VERSION_BCRUNCH);

		printf("Conversion de %s...\n", parametres.tabTypes[0].front().c_str());

		forever
		{
			try
			{
				fluxSortie->ecrireOctet(fluxEntree->lireOctet());
			}
			catch (EFinFichier &)
			{
				break;
			}
		}

	}

	void Commande::tester(ParametresCommande &parametres)
	{
		FluxBinaire *fluxEntree = new FluxBinaire(parametres.tabTypes[0].front().c_str(), LECTURE);

		if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
		{
			fluxEntree->aller(TAILLE_EXE_WIN32);

			if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
			{
				fluxEntree->aller(TAILLE_EXE_LINUX);

				if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
				{
					fluxEntree->aller(TAILLE_EXE_BSD);

					if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
					{
						fluxEntree->aller(TAILLE_EXE_MACOS);

						if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
						{
							printf("Format d'archive incompatible avec Big Crunch, impossible de tester\n");
							throw ENTETE_BCRUNCH;
						}
					}
				}
			}
		}

		byte version = fluxEntree->lireOctet();

		if (version != VERSION_BCRUNCH)
		{
			if (version < VERSION_BCRUNCH)
			{
				printf("Cette archive a ete compressee avec une ancienne version de BigCrunch (%ui), impossible de tester\n", version);
			}
			else
			{
				printf("Cette archive a ete compressee avec une nouvelle version de BigCrunch (%ui), impossible de tester\n", version);
			}
			throw VERSION_BCRUNCH;
		}

		bool corrompu = false;

		forever
		{

			try
			{

				string nomFichier = "";
				U8 caractere;

				while ((caractere = fluxEntree->lireOctet()) > 31)
				{
					nomFichier += caractere;
				}

				printf("Verification de %s...\n", nomFichier.c_str());

				if (caractere != ID)
				{

					uint tailleFichierCompresse = *((uint *)fluxEntree->lireOctets(sizeof(uint)));

					char *nomTmp = Fichier::NomFichierTemporaire();

					FluxBinaire *fluxFichierTemporaire = new FluxBinaire(nomTmp, ECRITURE);

					for (uint i = 0; i < tailleFichierCompresse; ++i)
					{
						U8 tampon = fluxEntree->lireOctet();
						fluxFichierTemporaire->ecrireOctet(tampon);
					}

					delete fluxFichierTemporaire;

					char *nomTmp2 = Fichier::NomFichierTemporaire();

					Codeur *codeur = NULL;
					//Appel du codeur en fonction du type de fichier
					switch (caractere)
					{
						case TXT :
							codeur = new ArSa1(nomTmp, DECOMPRESSER, nomTmp2);
							printf("Algorithme: ArSa1                          0.0%%");
							break;
						case EXE :
							/*codeur = new ArBin(nomTmp, DECOMPRESSER, nomTmp2);
							printf("Algorithme: ArBin                          0.0%%");
							break;*/
						case UKN :
							codeur = new ArFaMix(nomTmp, DECOMPRESSER, nomTmp2);
							printf("Algorithme: ArFaMix                        0.0%%");
							break;
						case BMP :
							codeur = new ArBMP(nomTmp, DECOMPRESSER, nomTmp2);
							printf("Algorithme: ArBMP                          0.0%%");
							break;
						case TGA :
							codeur = new ArTGA(nomTmp, DECOMPRESSER, nomTmp2);
							printf("Algorithme: ArTGA                          0.0%%");
							break;
						case INC :
							codeur = new Copie(nomTmp, DECOMPRESSER, nomTmp2);
							printf("Copie du fichier source (incompressible)   0.0%%");
							break;
						default :
							printf("Erreur endroit non logique\n");
					}
					codeur->decompresser();
					delete codeur;

					// On supprime le fichier temporaire
					Fichier::supprimer(nomTmp);

					printf("\b\b\b\b\b\b100.0%%\n");


					// TODO Jems : comparer les CRC32
					if (fluxEntree->lireU32() != Fichier::Crc32(nomTmp2))
					{
						printf("Erreur de controle de redondance cyclique\n");
						printf("Le fichier %s est corrompu\n", nomFichier.c_str());
						corrompu = true;
					}

					Fichier::supprimer(nomTmp2);
				}
			}
			catch (EFinFichier &)
			{
				break;
			}
		}

		if (corrompu)
		{
			printf("L'archive est corrompue\n");
		}
		else
		{
			printf("L'archive est integre\n");
		}

		if(parametres.cloche)
		{
			printf("\a");
		}
	}

#endif

}
