/*
** Big Crunch
** Compresseur exploitable et exprimental
**
** Copyright (C) 2005-2006 - KaBoom
**
** http://bcrunch.online.fr/
*/

#include"Parseur.hpp"

namespace BigCrunch
{

	bool Parseur::parser(int nbMots, char *commande[], ParametresCommande &parametres)
	{


		bool erreur = false;
#ifndef SFX
		bool parseOptions = true;

		if (nbMots < 2)
		{
			throw ESansArgument();
		}

		if (commande[1][1] != '\0')
		{
			printf("Commande inconnue %s\n", commande[1]);
			erreur = true;
		}
		else
		{
			switch (commande[1][0])
			{
				case 'a':
					parametres.action = AJOUTER;
					break;
				case 'x':
#endif
					parametres.action = EXTRAIRE;
#ifndef SFX
					break;
				case 'r':
					parametres.action = SUPPRIMER;
					break;
				case 'l':
					parametres.action = LISTER;
					break;
				case 'c':
					parametres.action = CONVERTIR;
					break;
				case 't':
					parametres.action = TESTER;
					break;
#ifdef DEBUG
				case 'i':
					return installer(nbMots, commande);
					break;
				case 'u':
					return desinstaller();
					break;
#endif
				default:
					printf("Commande inconnue %s\n", commande[1]);
					erreur = true;
					break;
			}
		}

		for (int i = 2; i < nbMots; ++i)
		{
			if (estEchappement(commande[i][0]))
			{
				parserFichier(commande[i] + 1, parametres, erreur);
			}
			else
			{
				if (parseOptions && estSwitch(commande[i][0]))
				{
					parserOption(commande[i] + 1, parametres, erreur, parseOptions);
				}
				else
				{
					parserFichier(commande[i], parametres, erreur);
				}
			}
		}
#else
					parserFichier(commande[2], parametres, erreur);
#endif

		return !erreur;
	}

#ifndef SFX
	void Parseur::parserOption(char *option, ParametresCommande &parametres, bool &erreur, bool &parseOptions)
	{
		if (strlen(option - 1) < 2)
		{
			printf("Option non specifiee\n");
			erreur = true;
			return;
		}
		switch (*option)
		{
			case '-':
				parseOptions = false;
				break;
			case 'u':
				parametres.miseAJour = true;
			case 'o':
				parserSortie(option + 1, parametres, erreur);
				break;
			case 'c':
				parserVolumes(option + 1, parametres, erreur);
				break;
			case 's':
				parserSfx(option + 1, parametres, erreur);
				break;
			case 'm':
				parserMode(option + 1, parametres, erreur);
				break;
			case 'a':
				parametres.optimiser = true;
				break;
			case 'k':
				parametres.forcer = true;
				break;
			case 'v':
				parametres.details = DETAILS_VERBOSE;
				break;
			case 'q':
				parametres.details = DETAILS_QUIET;
				break;
			case 'd':
				parserDetails(option + 1, parametres, erreur);
				break;
			case 'l':
				parserLog(option + 1, parametres, erreur);
				break;
			case 'b':
				parametres.cloche = true;
				break;
			case 'e':
				parametres.effacer = true;
				break;
			case 'x':
				parserReaction(option + 1, parametres, erreur);
				break;
			default:
				printf("Option inconnue %s\n", option - 1);
				erreur = true;
				break;
		}
	}

	void Parseur::parserSfx(char *sfx, ParametresCommande &parametres, bool &erreur)
	{
		if ((parametres.action == SUPPRIMER) || (parametres.action == LISTER) || (parametres.action == TESTER))
		{
			printf("Impossible de creer une archive autoextractible pour l'action demandee\n");
			erreur = true;
			return;
		}

		if (strlen(sfx - 1) < 2)
		{
			printf("Plate-forme de l'autoextractible non specifiee\n");
			erreur = true;
			return;
		}

		switch (*sfx)
		{
			case 'w':
				parametres.plateForme = WIN;
				break;
			case 'l':
				parametres.plateForme = LINUX;
				break;
			case 'b':
				parametres.plateForme = BSD;
				break;
			case 'm':
				parametres.plateForme = MACOS;
				break;
			default:
				printf("Plate-forme de l'autoextractible inconnue\n");
				erreur = true;
				return;
		}

	}


	void Parseur::parserSortie(char *fichier, ParametresCommande &parametres, bool &erreur)
	{
		if ((parametres.action == SUPPRIMER) || (parametres.action == LISTER) || (parametres.action == TESTER))
		{
			printf("Impossible de specifier la sortie pour l'action demandee\n");
			erreur = true;
			return;
		}

		if (strlen(fichier - 1) < 2)
		{
			printf("Fichier de sortie non specifie\n");
			erreur = true;
			return;
		}


		if (fichier[-1] == 'o')
		{
			if (Fichier::Existe(fichier))
			{
				printf("Sortie deja existante %s\n", fichier);
				erreur = true;
				return;
			}
		}
		else
		{
			if (!Fichier::Existe(fichier) || !Fichier::EstArchive(fichier))
			{
				printf("Impossible de mettre a jour %s\n", fichier);
				erreur = true;
				return;
			}
		}

		parametres.nomSortie = fichier;
	}

#endif

	void Parseur::parserFichier(char *fichier, ParametresCommande &parametres, bool &erreur)
	{
		if (parametres.action == AJOUTER)
		{
			if (Fichier::Existe(fichier))
			{
				parametres.fichiers->AddFile(fichier);
			}
			else
			{
				printf("Fichier innexistant %s\n", fichier);
				erreur = true;
				return;
			}
		}
		else
		{
			parametres.fichiers->AddForce(fichier);
		}
	}

#ifndef SFX


	void Parseur::parserVolumes(char *taille, ParametresCommande &parametres, bool &erreur)
	{
		parametres.tailleVolumes = atoi(taille);
		if (parametres.tailleVolumes == 0)
		{
			parametres.tailleVolumes = MAX_U32;
			printf("Taille de volumes improbable %s\n", taille - 1);
			erreur = true;
		}
	}

	void Parseur::parserMode(char *mode, ParametresCommande &parametres, bool &erreur)
	{
		switch (*mode)
		{
			case '0':
				parametres.mode = 0;
				break;
			case '1':
				parametres.mode = 1;
				break;
			case '2':
				parametres.mode = 2;
				break;
			case '3':
				parametres.mode = 3;
				break;
			case 'x':
				parametres.mode = 4;
				break;
			default:
				printf("Mode inconnu %s\n", mode - 1);
				erreur = true;
				break;
		}
	}

	void Parseur::parserDetails(char *details, ParametresCommande &parametres, bool &erreur)
	{
		switch (*details)
		{
			case '0':
				parametres.details = 0;
				break;
			case '1':
				parametres.details = 1;
				break;
			case '2':
				parametres.details = 2;
				break;
			case '3':
				parametres.details = 3;
				break;
			case '4':
				parametres.details = 4;
				break;
			case '5':
				parametres.details = 5;
				break;
			case '6':
				parametres.details = 6;
				break;
			default:
				printf("Niveau de details inconnu %s\n", details - 1);
				erreur = true;
				break;
		}
	}

	void Parseur::parserLog(char *log, ParametresCommande &parametres, bool &erreur)
	{
		parametres.logger = true;

		if (strlen(log - 1) < 2)
		{
			return;
		}

		if (Fichier::Existe(log))
		{
			printf("Sortie deja existante %s\n", log);
			erreur = true;
			return;
		}

		parametres.log = log;
	}


	void Parseur::parserReaction(char *reaction, ParametresCommande &parametres, bool &erreur)
	{
		switch (*reaction)
		{
			case '0':
				parametres.reaction = PASSER;
				break;
			case '1':
				parametres.reaction = RENOMMER;
				break;
			case '2':
				parametres.reaction = ECRASER;
				break;
			default:
				printf("Reaction inconnue %s\n", reaction - 1);
				erreur = true;
				break;
		}
	}

	bool Parseur::estSwitch(char caractere)
	{
		return caractere == '-' || caractere == '/';
	}



	bool Parseur::estEchappement(char caractere)
	{
		return caractere == '\\';
	}


	bool Parseur::installer(int nbMots, char *commande[])
	{
		if (nbMots == 3)
		{
			Langue langue = FR;
			nbMots = 2;

			if ((commande[2][0] != '-') || (commande[2][3] != '\0'))
			{
				return false;
			}

			if ((commande[2][1] == 'f') && (commande[2][2] == 'r'))
			{}
			else if ((commande[2][1] == 'e') && (commande[2][2] == 'n'))
			{
				langue = EN;
			}
			else if ((commande[2][1] == 'd') && (commande[2][2] == 'e'))
			{
				langue = DE;
			}
			else
			{
				return false;
			}
		}
		if (nbMots == 2)
		{
	#ifdef _WIN32_
			// TODO ajouter BigCrunch au path
			// TODO ajouter BigCrunch au shell
			// TODO ajouter BigCrunch au registre
	#endif
	#ifdef _LINUX_
			// TODO ajouter BigCrunch au path
	#endif
	#ifdef _BSD_
			// TODO ajouter BigCrunch au path
	#endif
	#ifdef _MACOS_
			// TODO ajouter BigCrunch au path
	#endif

			return true;
		}

		return false;
	}

	bool Parseur::desinstaller()
	{
#ifdef _WIN32_
		// TODO retirer BigCrunch du path
		// TODO retirer BigCrunch du shell
		// TODO retirer BigCrunch du registre
#endif
#ifdef _LINUX_
		// TODO retirer BigCrunch du path
#endif
#ifdef _BSD_
		// TODO retirer BigCrunch du path
#endif
#ifdef _MACOS_
		// TODO retirer BigCrunch du path
#endif

		return true;
	}

#endif
}
