/*
** Big Crunch
** Compresseur exploitable et exprimental
**
** Copyright (C) 2005-2006 - KaBoom
**
** http://bcrunch.online.fr/
*/

#include"Fichier.hpp"

namespace BigCrunch
{

	Fichier::Fichier(const char *nomFichier, ModeFlux mode)
	{
		_nomFichier = new char[strlen(nomFichier) + 1];
		strcpy(_nomFichier, nomFichier);

		if (mode == ECRITURE)
		{
			_flux = new FluxBinaire(nomFichier, ECRITURE);
		}
		else
		{
			_flux = NULL;
		}
	}


	Fichier::~Fichier()
	{
		if (_flux != NULL)
		{
			delete _flux;
		}
	}



	void Fichier::copier(const char *nomEntree)
	{
		FluxBinaire entree(nomEntree, LECTURE);

		forever
		{
			try
			{
				U8 tampon = entree.lireOctet();
				_flux->ecrireOctet(tampon);
			}
			catch (EFinFichier &)
			{
				break;
			}
		}
	}

	void Fichier::copier(const char *nomEntree, const char *nomSortie)
	{
		if (Existe(nomEntree))
		{
			FluxBinaire entree(nomEntree, LECTURE);
			FluxBinaire sortie(nomSortie, ECRITURE);

			forever
			{
				try
				{
					U8 tampon = entree.lireOctet();
					sortie.ecrireOctet(tampon);
				}
				catch (EFinFichier &)
				{
					break;
				}
			}
		}
	}



	void Fichier::renommer(const char *nouveauNom)
	{
		rename(_nomFichier, nouveauNom);

		delete [] _nomFichier;

		_nomFichier = new char[strlen(nouveauNom) + 1];
		strcpy(_nomFichier, nouveauNom);
	}


	void Fichier::renommer(const char *ancienNom, const char *nouveauNom)
	{
		rename(ancienNom, nouveauNom);
	}



	char *Fichier::changerExtension(const char *nomEntree, const char *extension)
	{
		int longueur = (int)strlen(nomEntree);

		char *nomFichier = new char[longueur + 5];
		char *base = new char[longueur + 1];
		strcpy(base, nomEntree);

		--longueur;
		while ((longueur > 0) && (nomEntree[longueur] != '.'))
		{
			--longueur;
		}
		if (longueur > 0)
		{
			base[longueur] = '\0';
		}

		sprintf(nomFichier, "%s%s", base, extension);

		base[longueur] = 'a';
		delete [] base;

		return nomFichier;
	}



	void Fichier::supprimer(const char *nomFichier)
	{
		remove(nomFichier);
	}

	void Fichier::supprimer()
	{
		remove(_nomFichier);
	}



	bool Fichier::Existe(const char *nomFichier)
	{
		try
		{
			FluxBinaire fichier(nomFichier, LECTURE);
		}
		catch (...)
		{
			return false;
		}

		return true;
	}



	bool Fichier::Existe()
	{
		try
		{
			FluxBinaire fichier(_nomFichier, LECTURE);
		}
		catch (...)
		{
			return false;
		}

		return true;
	}



	uint Fichier::Taille()
	{
		FluxBinaire fichier(_nomFichier, LECTURE);
		return fichier.Taille();
	}

	uint Fichier::Taille(const char *nomFichier)
	{
		FluxBinaire fichier(nomFichier, LECTURE);
		return fichier.Taille();
	}


	uint Fichier::Crc32()
	{
		return Crc32::hash(_nomFichier);
	}


	uint Fichier::Crc32(const char *nomFichier)
	{
		return Crc32::hash(nomFichier);
	}


	char *Fichier::NomFichierTemporaire()
	{
		char *nomFichier = new char[10];

		sprintf(nomFichier, "bc0.tmp");

		uint i = 1;
		FILE *test = NULL;
		while ((i < 1000) && ((test = fopen(nomFichier, "rb")) != NULL))
		{
			fclose(test);
			sprintf(nomFichier, "bc%u.tmp", i);
			++i;
		}

		return nomFichier;
	}

	char *Fichier::Nommer(const char *idee)
	{
		int longueur = (int)strlen(idee);

		char *nomFichier = new char[longueur + 6];
		char *base = new char[longueur + 1];
		strcpy(base, idee);
		char *copie = new char[longueur + 1];
		strcpy(copie, idee);
		char *extension;

		--longueur;
		while ((longueur > 0) && (idee[longueur] != '.'))
		{
			--longueur;
		}
		if (longueur <= 0)
		{
			extension = "";
		}
		else
		{
			extension = copie + longueur;
			base[longueur] = '\0';
		}

		sprintf(nomFichier, "%s[1]%s", base, extension);

		uint i = 2;
		FILE *test = NULL;
		while ((i < 1000) && ((test = fopen(nomFichier, "rb")) != NULL))
		{
			fclose(test);
			sprintf(nomFichier, "%s[%u]%s", base, i, extension);
			++i;
		}

		base[longueur] = 'a';
		delete [] base;
		delete [] copie;

		return nomFichier;
	}


	bool Fichier::EstArchive(const char *nomFichier)
	{
		FluxBinaire *fluxEntree = new FluxBinaire(nomFichier, LECTURE);

		if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
		{
			fluxEntree->aller(TAILLE_EXE_WIN32);

			if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
			{
				fluxEntree->aller(TAILLE_EXE_LINUX);

				if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
				{
					fluxEntree->aller(TAILLE_EXE_BSD);

					if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
					{
						fluxEntree->aller(TAILLE_EXE_MACOS);

						if (fluxEntree->lireOctet() != ENTETE_BCRUNCH)
						{
							return false;
						}
					}
				}
			}
		}

		if (fluxEntree->lireOctet() != VERSION_BCRUNCH)
		{
			return false;
		}

		return true;
	}


	FluxBinaire *Fichier::GetFlux()
	{
		return _flux;
	}



}
