/*
** Big Crunch
** Compresseur exploitable et exprimental
**
** Copyright (C) 2005-2006 - KaBoom
**
** http://bcrunch.online.fr/
*/

#include"FluxBinaire.hpp"

//#include"ParcDossier.hpp"

namespace BigCrunch
{


	// Cre une exception fin de fichier avec nbOctetsLus octets lus
	EFinFichier::EFinFichier(byte nbOctetsLus = 0)
	{
		_nbOctetsLus = nbOctetsLus;
	}



	// Retourne le nombre d'octets lus
	char EFinFichier::NbOctetsLus()
	{
		return _nbOctetsLus;
	}



	// Cre un nouveau flux binaire et le lie au fichier
	FluxBinaire::FluxBinaire(const char *nomFichier, ModeFlux mode)
	{
		// On garde le nom du fichier
		_nomFichier = new char[strlen(nomFichier) + 1];
		strcpy(_nomFichier, nomFichier);

		// Suivant le mode spcifi, on ouvre le fichier en lecture ou en criture
		if (mode == LECTURE)
		{
			_fichier = fopen(nomFichier, "rb");

			// On met un marqueur dans le tampon pour savoir quand il est vide
			_tampon = 256;
		}
		else
		{
		//	string tmp = nomFichier;

		//	Dossier::mkPath(tmp);

			_fichier = fopen(nomFichier, "ab");

			// On met un marqueur dans le tampon pour savoir quand il est plein
			_tampon = 1;
		}

		// Si le fichier d'entre n'existe pas on dclenche une erreur
		if (!_fichier)
		{
			/* TODO (Jems#1#): envoyer une exception avec un chti message */
			throw nomFichier;
		}
	}



	// Dtruit le flux et libre le fichier
	FluxBinaire::~FluxBinaire()
	{
		fclose(_fichier);
	}



	// Ecrit le bit dans le fichier (suppose que bit a pour valeur 0 ou 1)
	void FluxBinaire::ecrireBit(U8 bit)
	{
		// On ajoute le bit au tampon
		_tampon <<= 1;
		_tampon += (bit & 1);

		// Si le tampon est plein, on le purge et on remet le marqueur
		if (_tampon > 255)
		{
			putc(_tampon, _fichier);
			_tampon = 1;
		}
	}



	void FluxBinaire::ecrireBits(void *bits, uint nbBits)
	{
		for (long i = nbBits - 1; i >= 0; --i)
		{
			ecrireBit((U8)((*((U64 *)bits) >> i) & 1));
		}
	}



	// Ecrit l'octet dans le fichier (mthode optimise s'il n'y a pas de dcalage)
	void FluxBinaire::ecrireOctet(U8 octet)
	{
		if (_tampon == 1)
		{
			// Si il n'y a pas de dcalage, on crit l'octet directement (rapide)
			putc(octet, _fichier);
		}
		else
		{
			// Sinon, on l'crit bit par bit (lent)
			for (int i = 7; i >= 0; --i)
			{
				ecrireBit((U8)((octet >> i) & 1));
			}
		}
	}


	void FluxBinaire::ecrireU16(U16 mot)
	{
		ecrireOctet((U8)(mot & 0xff));
		ecrireOctet((U8)(mot >> 8));
	}


	void FluxBinaire::ecrireU32(U32 doubleMot)
	{
		ecrireOctet((U8)(doubleMot & 0xff));
		ecrireOctet((U8)((doubleMot >> 8) & 0xff));
		ecrireOctet((U8)((doubleMot >> 16) & 0xff));
		ecrireOctet((U8)(doubleMot >> 24));
	}


	// Ecrit nbOctets octets dans le fichier (mthode optimise s'il n'y a pas de dcalage)
	void FluxBinaire::ecrireOctets(void *octets, uint nbOctets)
	{
		U8 *P;
		P =(U8 *) octets;
		for (uint i = 0; i < nbOctets; ++i)
		{
			ecrireOctet(P[i]);
		}
	}


	// Vide le tampon dans le fichier, pour ne pas oublier les bits orphelins
	void FluxBinaire::purger()
	{
		// On aligne le tampon
		while (_tampon <= 255)
		{
			_tampon <<= 1;
		}

		// On le purge
		putc(_tampon, _fichier);

		// On remet le marqueur dans le tampon pour savoir quand il est plein
		_tampon = 1;
	}



	// Lit un bit dans le fichier
	U8 FluxBinaire::lireBit()
	{
		// Si le tampon est vide, on le remplit
		if (_tampon == 256)
		{
			U8 tampon = (U8)(getc(_fichier));

			if (feof(_fichier))
			{
				throw EFinFichier(0);
			}
			_tampon = tampon;
			_tampon <<= 8;
			_tampon += 1;
		}

		// On rcupre le bit de poids fort
		U8 bit = ((U8)(_tampon >> 15));

		_tampon <<= 1;

		return bit;
	}



	void *FluxBinaire::lireBits(uint nbBits)
	{
		U64 *tampon = new U64;
		*tampon = 0;

		for (uint i = 0; i < nbBits; ++i)
		{
			*tampon <<= 1;
			*tampon |= (lireBit() & 1);
		}

		return tampon;
	}



	// Lit un octet dans le fichier (mthode optimise s'il n'y a pas de dcalage)
	U8 FluxBinaire::lireOctet()
	{
		if (_tampon == 256)
		{
			U8 tampon = (U8)(getc(_fichier));

			// Si il n'y a pas de dcalage, on lit l'octet directement (rapide)
			if (feof(_fichier))
			{
				throw EFinFichier(0);
			}
			return tampon;
		}
		else
		{
			// Sinon, on le lit bit par bit (lent)
			U8 octet = 0;
			for (int i = 7; i >= 1; --i)
			{
				octet |= lireBit();
				octet <<= 1;
			}
			return octet | lireBit();
		}
	}



	void *FluxBinaire::lireOctets(uint nbOctets)
	{
		U8 *octets = new U8[nbOctets];

		for (uint i = 0; i < nbOctets; ++i)
		{
			if (feof(_fichier))
			{
				throw EFinFichier((byte)i);
			}
			octets[i] = lireOctet();
		}

		return octets;
	}

	U16 FluxBinaire::lireU16()
	{
		U8 mot[2];

		for (byte i = 0; i < 2; ++i)
		{
			if (feof(_fichier))
			{
				throw EFinFichier(i);
			}
			mot[i] = lireOctet();
		}

		return *((U16 *)mot);
	}

	U32 FluxBinaire::lireU32()
	{
		U8 mot[4];

		for (byte i = 0; i < 4; ++i)
		{
			if (feof(_fichier))
			{
				throw EFinFichier(i);
			}
			mot[i] = lireOctet();
		}

		return *((U32 *)mot);
	}



	// Vide le flux (sans le dtruire)
	void FluxBinaire::vider()
	{
		fclose(_fichier);
		_fichier = fopen(_nomFichier, "wb");

		// On met un marqueur dans le tampon pour savoir quand il est plein
		_tampon = 1;
	}



	// Revient au dbut du flux
	void FluxBinaire::rembobiner()
	{
		fseek(_fichier, 0, SEEK_SET);

		// On met un marqueur dans le tampon pour savoir quand il est vide
		_tampon = 256;
	}


	// Va  la fin du flux
	void FluxBinaire::debobiner()
	{
		fseek(_fichier, 0, SEEK_END);

		// On met un marqueur dans le tampon pour savoir quand il est vide
		_tampon = 256;
	}

	void FluxBinaire::rembobiner(int de)
	{
		fseek(_fichier, -de, SEEK_CUR);

		// On met un marqueur dans le tampon pour savoir quand il est vide
		_tampon = 256;
	}


	void FluxBinaire::avancer(int de)
	{
		fseek(_fichier, de, SEEK_CUR);

		// On met un marqueur dans le tampon pour savoir quand il est vide
		_tampon = 256;
	}


	void FluxBinaire::aller(int a)
	{
		fseek(_fichier, a, SEEK_SET);
	}


	// Retourne la taille du flux
	uint FluxBinaire::Taille()
	{
		uint pos = ftell(_fichier);

		fseek(_fichier, 0, SEEK_END);
		uint taille = ftell(_fichier);

		fseek(_fichier, pos, SEEK_SET);

		return taille;
	}



	// Retourne la position dans le flux
	uint FluxBinaire::Position()
	{
		return ftell(_fichier);
	}


	char *FluxBinaire::NomFichier()
	{
		return _nomFichier;
	}

}
