/*
** Big Crunch
** Compresseur exploitable et exprimental
**
** Copyright (C) 2005-2006 - KaBoom
**
** http://bcrunch.online.fr/
*/

/*
** Classe FluxBinaire
**
** 2005-12-27	Arkanosis	lireOctets, ecrireOctets, vider, Taille, Position
** 2005-11-28	Arkanosis   Classe, prototypes, premire implmentation
**
** TODO : ---
*/

/*############################*/
#ifndef FLUX_BINAIRE_HPP
#	define FLUX_BINAIRE_HPP
/*############################*/

#include"../definitions.hpp"

namespace BigCrunch
{

	// Mode de fonctionnement du flux (lecture ou criture)
	enum ModeFlux
	{
		LECTURE,
		ECRITURE
	};



	/*
	** Description :
	** 	Une exception EFinFichier prvient que la lecture n'a pas pu tre faite correctement.
	** 	La proprit NbOctetsLus contient le nombre d'octets qui ont ventuellement pu tre lus.
	**
	** Constructeurs :
	** 	- EFinFichier(nbOctetsLus)
	** 		Cre une exception fin de fichier avec nbOctetsLus octets lus
	**
	** Mthodes :
	**
	** Proprits :
	** 	- Taille()
	** 		Retourne la taille du flux
	*/
	class EFinFichier
	{
		public:
			// Constructeurs
			EFinFichier(byte nbOctetsLus);
			// Mthodes
			// Proprits
			char NbOctetsLus();
		protected:
			// Champs
			char _nbOctetsLus;
	};



	/*
	** Description :
	** 	Un flux binaire permet d'crire ou de lire dans un fichier bit par bit.
	**
	** Constructeurs :
	** 	- FluxBinaire(char *fichier, Mode mode)
	** 		Cre un nouveau flux binaire et le lie au fichier,
	** 		suivant le mode : en LECTURE ou ECRITURE
	** 	- ~FluxBinaire()
	** 		Dtruit le flux et libre le fichier
	**
	** Mthodes :
	** 	- void ecrireBit(U8 bit)
	** 		Ecrit le bit dans le fichier
	** 	- void EcrireOctet(U8 octet)
	** 		Ecrit l'octet dans le fichier
	** 	- void EcrireOctets(U8 *octets, int nbOctets)
	** 		Ecrit nbOctets octets dans le fichier
	** 	- void purger()
	** 		Doit tre appele  la fin de toute criture (avant de dtruire le flux)
	** 	- U8 lireBit()
	** 		Lit un bit dans le fichier et le retourne
	** 	- U8 lireOctet()
	** 		Lit un octet dans le fichier et le retourne
	** 	- U8 lireOctets(int nbOctets)
	** 		Lit nbOctets octets dans le fichier et les retourne
	** 	- void rembobiner()
	** 		Revient au dbut du fichier
	** 	- void vider()
	** 		Vide le flux (sans le dtruire)
	**
	** Proprits :
	** 	- int Taille()
	** 		Retourne la taille du flux
	** 	- int Position()
	** 		Retourne la position dans le flux
	*/

	class FluxBinaire
	{
		public:
			// Constructeurs
			FluxBinaire(const char *nomFichier, ModeFlux mode);
			~FluxBinaire();
			// Mthodes
			void ecrireBit(U8 bit);
			void ecrireBits(void *bits, uint nbBits);
			void ecrireOctet(U8 octet);
			void ecrireU16(U16 mot);
			void ecrireU32(U32 doubleMot);
			void ecrireOctets(void *octets, uint nbOctets);
			void purger();
			U8   lireBit();
			void *lireBits(uint nbBits);
			U8   lireOctet();
			U16 FluxBinaire::lireU16();
			U32 FluxBinaire::lireU32();
			void *lireOctets(uint nbOctets);
			void vider();
			void rembobiner();
			void rembobiner(int de);
			void debobiner();
			void avancer(int de);
			void aller(int a);
			// Proprits
			uint  Taille();
			uint  Position();
			char *NomFichier();


		protected:
			// Champs
			char *_nomFichier;
			FILE *_fichier;
			U16  _tampon;
	};

}

/*############################*/
#endif
/*############################*/
