/*
** Big Crunch
** Compresseur exploitable et exprimental
**
** Copyright (C) 2005-2006 - KaBoom
**
** http://bcrunch.online.fr/
*/

#include"Dico.hpp"

namespace BigCrunch
{

	Dico::Dico(byte tailleCode)
	{
		_codeSuivant = 256;

		_tailleCode = tailleCode;

		switch (_tailleCode)
		{
			/* Les premiers cas ne sont pas interessants pour la compression
			case 9:
				_tailleTable = 641; // 60 Ko
				break;
			case 10:
				_tailleTable = 1283; // 130 Ko
				break;*/
			case 11:
				_tailleTable = 2579; // 260 Ko
				break;
			case 12:
				_tailleTable = 5147; // 520 Ko
				break;
			case 13:
				_tailleTable = 10243; // 1 Mo
				break;
			case 14:
				_tailleTable = 20483; // 2 Mo
				break;
			case 15:
				_tailleTable = 40961; // 4 Mo
				break;
			case 16:
				_tailleTable = 81929; // 8 Mo
				break;
			case 17:
				_tailleTable = 163841; // 17 Mo
				break;
			case 18:
				_tailleTable = 327689; // 33 Mo
				break;
			case 19:
				_tailleTable = 655373; // 66 Mo
				break;
			case 20:
				_tailleTable = 1310723; // 130 Mo
				break;
			case 21:
				_tailleTable = 2621447; // 262 Mo
				break;
			case 22:
				_tailleTable = 5242883; // 524 Mo
				break;
			case 23:
				_tailleTable = 10485767; // 1 Go
				break;
			case 24:
				_tailleTable = 20971529; // 2 Go
				break;
			case 25:
				_tailleTable = 41943049; // 4 Go
				break;
			// Les cas suivants ncessitent trop de mmoire actuellement
			/*case 26:
				_tailleTable = 83886091; // 8 Go
				break;
			case 27:
				_tailleTable = 167772161; // 17 Go
				break;
			case 28:
				_tailleTable = 292594649; // 29 Go
				break;
			case 29:
				_tailleTable = 542239657; // 54 Go
				break;
			case 30:
				_tailleTable = 1084479271; // 108 Go
				break;
			case 31:
				_tailleTable = 2168958485; // 217 Go
				break;*/
			default:
				// TODO grer toutes les tailles avec des nombres 1ers
				_tailleTable = 0;
				break;
		}

		_codeMaximum = (1 << _tailleCode) - 1; // TODO a t modifi ! (original : "_codeMaximum = (1 << _tailleCode) - 2")

		_decalageHash = _tailleCode - 8;

		_table = new MotDico[_tailleTable];

		// TODO Transformer en exception
		if (!_table)
		{
			printf("Impossible d'allouer l'espace memoire necessaire\n");
			exit(1);
		}

		for (uint i = 0; i < _tailleTable; ++i)
		{
			_table[i].code = -1;
		}
	}

	Dico::~Dico()
	{
		delete [] _table;
	}

	bool Dico::ajouter(U32 index, U32 codeChaine, U8 caractere)
	{
		printf("%i - %i\n", codeChaine, caractere);

		if (_codeSuivant <= _codeMaximum)
		{
			_table[index].code      = _codeSuivant;
			_table[index].prefixe   = codeChaine;
			_table[index].caractere = caractere;

			++_codeSuivant;

			return true;
		}

		return false;
	}



	void Dico::purger()
	{
		_codeSuivant = 256;

		for (uint i = 0; i < _tailleTable; ++i)
		{
			_table[i].code = -1;
		}
	}


	U32 Dico::getIndex(U32 codeChaine, U8 caractere)
	{
		int pas;

		// Fonction de hash basique, on peut peut-tre faire mieux
		int hash = (caractere << _decalageHash) ^ codeChaine;

		if (hash == 0)
		{
			pas = 1;
		}
		else
		{
			pas = _tailleTable - hash;
		}

		forever
		{
			if ((_table[hash].code == -1) || ((_table[hash].prefixe == codeChaine) && (_table[hash].caractere == caractere)))
			{
				return(hash);
			}
			else
			{
				hash -= pas;

				if (hash < 0)
				{
					hash += _tailleTable;
				}
			}
		}
	}

	U32 Dico::getCodeSuivant()
	{
		return _codeSuivant;
	}


	bool Dico::Existe(U32 index)
	{
		return _table[index].code != -1;
	}

	bool Dico::EstPlein()
	{
		return _codeSuivant > _codeMaximum;
	}

}

