/*
** Big Crunch
** Compresseur exploitable et exprimental
**
** Copyright (C) 2005-2006 - KaBoom
**
** http://bcrunch.online.fr/
*/

#include"Lzr.hpp"

#define LZR_CSTE_TAILLE_CODE_DEFAUT 18
#define LZR_CSTE_CODE_MAX_DEFAUT 0x3ffff // %111111111111111111

#define LZR_CSTE_DICO_VIDE 0
#define LZR_CSTE_DICO_PLEIN 1

namespace BigCrunch
{

	Lzr::Lzr(const char *nomFichier, ModeCodeur mode) : Codeur(nomFichier, mode)
	{
		_tailleCodeMax = LZR_CSTE_TAILLE_CODE_DEFAUT;
		_codeMax = LZR_CSTE_CODE_MAX_DEFAUT;
	}

	Lzr::Lzr(const char *nomFichier, ModeCodeur mode, const char *nomSortie) : Codeur(nomFichier, mode, nomSortie)
	{
		_tailleCodeMax = LZR_CSTE_TAILLE_CODE_DEFAUT;
		_codeMax = LZR_CSTE_CODE_MAX_DEFAUT;

		// TODO Debug BitsFlat
		/*printf("\ngo\n");
		_tailleCodeMax = 22;
		U32 deb = 425;
		U32 mx = 578;


		#if 0
			_sortie = new FluxBinaire("test", ECRITURE);
			ecrireBitsFlat(deb, mx);
			_sortie->purger();
		#else
			_entree = new FluxBinaire("test", LECTURE);
			U32 res = lireBitsFlat(mx);

			if (res == deb)
				printf("\nOK\n");
			else
				printf("\nPB\n");
		#endif*/
	}


#ifndef SFX
	void Lzr::preparer()
	{
		// TODO (Jems#3#) Changer la taille du dictionnaire permet d'amliorer la compression
		_tailleCodeMax = 10; // TODO debug, remettre  22
		_codeMax = (1 << _tailleCodeMax) - 1;
	}



	const char *Lzr::compresser(bool preparer)
	{
		if (preparer)
		{
			this->preparer();
		}

		_sortie->ecrireOctet(_tailleCodeMax);

		_dicoVide  = new Dico(_tailleCodeMax);
		_dicoPlein = new Dico(_tailleCodeMax);

		U8 caractere        = _entree->lireOctet();
		U32 codeChaineVide  = caractere;
		U32 codeChainePlein = caractere;
		U32 dbgDerniereChaineAjoute = caractere;
		U32 dbgDernierCaractereAjoute = caractere;

		U32 index;
		bool dansVide  = true;
		bool dansPlein = true;

		// On remplit le premier dictionnaire
		forever
		{
			try
			{
				afficherProgression();

				caractere = _entree->lireOctet();

				index = _dicoPlein->getIndex(codeChainePlein, caractere);
				if (_dicoPlein->Existe(index))
				{
					codeChainePlein = (*_dicoPlein)[index].code;
				}
				else
				{
					ecrireBitsFlat(codeChainePlein, _dicoPlein->getCodeSuivant());

					if (_dicoPlein->getCodeSuivant() == _codeMax)
					{
						break;
					}
					else
					{
						_dicoPlein->ajouter(index, codeChainePlein, caractere);
						dbgDerniereChaineAjoute = codeChainePlein;
						dbgDernierCaractereAjoute = caractere;
					}

					codeChainePlein = caractere;
				}
			}
			catch (EFinFichier &)
			{
				ecrireBitsFlat(codeChainePlein, _dicoPlein->getCodeSuivant());

				_sortie->purger();

				return _nomSortie;
			}
		}

		//printf("\nDernier code : %i, derniere chaine : %i, dernier caractere : %i, caractere courant : %i\n", _dicoPlein->getCodeSuivant() - 1, dbgDerniereChaineAjoute, dbgDernierCaractereAjoute, caractere);

		// On prcise qu'on a fini de remplir le premier dictionnaire
		ecrireBitsFlat(_codeMax, _dicoPlein->getCodeSuivant());

		_sortie->ecrireOctet(caractere); // debug

		printf("\nYOOOOO\n"); // TODO enlever

		// On continue la compression, mais avec les deux dictionnaires en parallle
		forever
		{
			try
			{
				afficherProgression();

				codeChaineVide  = caractere;
				codeChainePlein = caractere;

				dansVide  = true;
				dansPlein = true;

				// Tant que la chaine est dans les deux dicos
				// (On progresse dans les deux en mme temps)
				while (dansVide && dansPlein)
				{
					caractere = _entree->lireOctet();

					// On avance dans le dico en cours de remplissage
					index = _dicoVide->getIndex(codeChaineVide, caractere);
					if (dansVide = _dicoVide->Existe(index))
					{
						codeChaineVide = (*_dicoVide)[index].code;
					}
					else
					{
						_dicoVide->ajouter(index, codeChaineVide, caractere);
					}

					// On avance dans le dico plein
					index = _dicoPlein->getIndex(codeChainePlein, caractere);
					if (dansPlein = _dicoPlein->Existe(index))
					{
						codeChainePlein = (*_dicoPlein)[index].code;
					}
				}

				// Tant que la chaine n'est que dans le dico en cours de remplissage
				// (On remplit le dico comme dans LZR2)
				if (dansVide)
				{
					do
					{
						caractere = _entree->lireOctet();

						// On avance dans le dico en cours de remplissage
						index = _dicoVide->getIndex(codeChaineVide, caractere);
						if (dansVide = _dicoVide->Existe(index))
						{
							codeChaineVide = (*_dicoVide)[index].code;
						}
						else
						{
							_dicoVide->ajouter(index, codeChaineVide, caractere);
						}
					} while (dansVide);

					// On crit le code correspondant  la chaine
					_sortie->ecrireBit(LZR_CSTE_DICO_VIDE);
					ecrireBitsFlat(codeChaineVide, _dicoVide->getCodeSuivant());

					// On rajuste la longueur des codes (des entres ont pu tre ajoutes)
					reajusteCompression();

					// On passe au caractre suivant
					continue;
				}

				// Tant que la chaine n'est que dans le dico plein
				// (On recopie toutes les chaines intermdiaires dans dicoVide)
				if (dansPlein)
				{
					do
					{
						caractere = _entree->lireOctet();

						// On avance dans le dico plein
						index = _dicoPlein->getIndex(codeChainePlein, caractere);
						if (dansPlein = _dicoPlein->Existe(index))
						{
							codeChainePlein = (*_dicoPlein)[index].code;
						}

						// On en profite pour enrichir le dico vide
						index = _dicoVide->getIndex(_dicoVide->getCodeSuivant() - 1, caractere);
						_dicoVide->ajouter(index, _dicoVide->getCodeSuivant() - 1, caractere);
					} while (dansPlein);

					// On crit le code correspondant  la chaine
					_sortie->ecrireBit(LZR_CSTE_DICO_PLEIN);
					_sortie->ecrireBits((void *)&codeChainePlein, _tailleCodeMax);

					// On rajuste la longueur des codes (des entres ont pu tre ajoutes)
					reajusteCompression();

					// On passe au caractre suivant
					continue;
				}

				// Quand la chaine n'est plus dans aucun dico
				// (On favorise le dico vide qui a des codes plus courts)
				_sortie->ecrireBit(LZR_CSTE_DICO_VIDE);
				ecrireBitsFlat(codeChaineVide, _dicoVide->getCodeSuivant());

				// On rajuste la longueur des codes (des entres ont pu tre ajoutes)
				reajusteCompression();
			}
			catch (EFinFichier &)
			{
				// On favorise le dico en cours de remplissage (codes plus courts)
				if (dansVide || !dansPlein)
				{
					_sortie->ecrireBit(LZR_CSTE_DICO_VIDE);
					ecrireBitsFlat(codeChaineVide, _dicoVide->getCodeSuivant());
				}
				else
				{
					_sortie->ecrireBit(LZR_CSTE_DICO_PLEIN);
					_sortie->ecrireBits((void *)&codeChainePlein, _tailleCodeMax);
				}

				_sortie->purger();

				return _nomSortie;
			}
		}

	}

	void Lzr::ecrireBitsFlat(U64 bits, U64 masque)
	{
		masque = 0xffffff; // TODO DEBUG SEULEMENT

		long i = _tailleCodeMax - 1;
		while (i >= 0)
		{
			if ((masque >> i) & 1)
			{
				byte bit = (bits >> i) & 1;
				if (!bit)
				{
					break;
				}
				_sortie->ecrireBit(bit);
			}
			--i;
		}
		_sortie->ecrireBits((void *)&bits, i + 1);
	}

#endif


	U8 *Lzr::decoderChaine(Dico *dictionnaire, U8 *tampon, U32 codeChaine)
	{
		while (codeChaine > 255)
		{
			*tampon    = (*dictionnaire)[codeChaine].caractere;
			codeChaine = (*dictionnaire)[codeChaine].prefixe;

			++tampon;
		}

		*tampon = codeChaine;

		return(tampon);
	}

	void Lzr::swapDicos()
	{
		printf("\nSWAAAP KILLAHHHH\n"); exit(1);
		Dico *dicoTmp = _dicoPlein;
		_dicoPlein = _dicoVide;
		_dicoVide = dicoTmp;
	}

	void Lzr::reajusteCompression()
	{
		if (_dicoVide->getCodeSuivant() > _codeMax)
		{
			// On enlve le dernier code du dico qu'on ne pourra pas retrouver  la dcompression
			(*_dicoVide)[_codeMax].code = -1;
			// On prvient que l'on reajuste
			_sortie->ecrireBit(LZR_CSTE_DICO_PLEIN);
			U64 tmp = 0;
			_sortie->ecrireBits((void *)&tmp, _tailleCodeMax);
			// On vide le dico plein, et on change les dicos
			_dicoPlein->purger();
			swapDicos();
		}
	}

	void Lzr::reajusteDecompression()
	{
			// On vide le dico plein, et on change les dicos
			_dicoPlein->purger();
			swapDicos();
	}

	U64 Lzr::lireBitsFlat(U64 masque)
	{
		masque = 0xffffff; // TODO DEBUG SEULEMENT

		U64 code = 0;

		long i = _tailleCodeMax - 1;
		while (i >= 0)
		{
			if ((masque >> i) & 1)
			{
				byte bit = _entree->lireBit();
				code <<= 1;
				code |= bit;
				if (!bit)
				{
					--i;
					break;
				}
			}
			else
			{
				code <<= 1;
			}
			--i;
		}
		while (i >= 0)
		{
			code <<= 1;
			code |= _entree->lireBit();
			--i;
		}

		return code;
	}



	const char *Lzr::decompresser()
	{
		_tailleCodeMax = _entree->lireOctet();
		_codeMax = (1 << _tailleCodeMax) - 1;

		_dicoVide  = new Dico(_tailleCodeMax);
		_dicoPlein = new Dico(_tailleCodeMax);

		U8 dico = LZR_CSTE_DICO_VIDE;
		U8 tamponChaine[8000];
		U8 *chaine;

		//_entree->lireBit(); // TODO juste pour LZRb !!!! enlever de toute urgence
		U8  caractere     = lireBitsFlat(_dicoPlein->getCodeSuivant());
		U32 codeChaine    = caractere;
		U32 codeChaineTmp = caractere;
		U32 dbgDerniereChaineAjoute = caractere;
		U32 dbgDernierCaractereAjoute = caractere;

		_sortie->ecrireOctet(caractere);

		// On remplit le premier dictionnaire
		forever
		{
			try
			{
				afficherProgression();

				codeChaine = lireBitsFlat(_dicoPlein->getCodeSuivant());

				if (codeChaine == _codeMax)
				{
					break;
				}

				if (codeChaine >= _dicoPlein->getCodeSuivant())
				{
					*tamponChaine = caractere;
					chaine = decoderChaine(_dicoPlein, tamponChaine + 1, codeChaineTmp);
				}
				else
				{
					chaine = decoderChaine(_dicoPlein, tamponChaine, codeChaine);
				}

				caractere = *chaine;

				while (chaine >= tamponChaine)
				{
					_sortie->ecrireOctet(*chaine);

					--chaine;
				}

				_dicoPlein->ajouter(_dicoPlein->getCodeSuivant(), codeChaineTmp, caractere);
				dbgDerniereChaineAjoute = codeChaineTmp;
				dbgDernierCaractereAjoute = caractere;

				codeChaineTmp = codeChaine;

			}
			catch (EFinFichier &)
			{
				break;
			}
		}

		//printf("\nDernier code : %i, derniere chaine : %i, dernier caractere : %i, caractere courant : %i\n", _dicoPlein->getCodeSuivant() - 1, dbgDerniereChaineAjoute, dbgDernierCaractereAjoute, caractere);

		printf("\nYOOOOO\n"); // TODO enlever

		caractere = _entree->lireOctet();//dbg
		codeChaineTmp = caractere;//dbg

		_sortie->ecrireOctet(caractere);//dbg

		// On continue la dcompression, mais avec les deux dictionnaires en parallle
		forever
		{
			try
			{
				afficherProgression();

				dico = _entree->lireBit();

				if (dico == LZR_CSTE_DICO_VIDE)
				{
					codeChaine = lireBitsFlat(_dicoVide->getCodeSuivant());

					if (codeChaine >= _dicoVide->getCodeSuivant())
					{
						*tamponChaine = caractere;
						chaine = decoderChaine(_dicoVide, tamponChaine + 1, codeChaineTmp);
					}
					else
					{
						chaine = decoderChaine(_dicoVide, tamponChaine, codeChaine);
					}

					caractere = *chaine;

					while (chaine >= tamponChaine)
					{
						_sortie->ecrireOctet(*chaine);

						--chaine;
					}

					_dicoVide->ajouter(_dicoVide->getCodeSuivant(), codeChaineTmp, caractere); // TODO vrif : pkoi codeSuivant ???

					codeChaineTmp = codeChaine;
				}
				else
				{

					codeChaine = *((U32 *)_entree->lireBits(_tailleCodeMax));

					if (codeChaine == 0)
					{
						reajusteDecompression();
						continue;
					}

					chaine = decoderChaine(_dicoPlein, tamponChaine, codeChaine);
					U8 *chaineTmp = chaine;

					while (chaine > tamponChaine)
					{
						_sortie->ecrireOctet(*chaine);

						--chaine;
					}
					_sortie->ecrireOctet(*chaine);

					// On ajoute les chaines intermdiaires dans dicoVide // TODO n'a-t-on pas un caractere d'avance ?
					while (chaine <= chaineTmp)
					{
						caractere = *chaine;

						_dicoVide->ajouter(_dicoVide->getCodeSuivant(), codeChaineTmp, caractere);
						codeChaineTmp = _dicoVide->getCodeSuivant() - 1; // Euh non spa ca ?

						++chaine;
					}
				}
			}
			catch (EFinFichier &)
			{
				break;
			}
		}

		return _nomSortie;
	}

}
